# A test suite for pdb; at the moment, this only validates skipping of
# specified test modules (RFE #5142).

import imp
import sys
import os
import unittest
import subprocess
import textwrap

from test import test_support
# This little helper class is essential for testing pdb under doctest.
from test_doctest import _FakeInput


class PdbTestCase(unittest.TestCase):

    def run_pdb(self, script, commands):
        """Run 'script' lines with pdb and the pdb 'commands'."""
        filename = 'main.py'
        with open(filename, 'w') as f:
            f.write(textwrap.dedent(script))
        self.addCleanup(test_support.unlink, filename)
        cmd = [sys.executable, '-m', 'pdb', filename]
        stdout = stderr = None
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                   stdin=subprocess.PIPE,
                                   stderr=subprocess.STDOUT,
                                   )
        stdout, stderr = proc.communicate(commands)
        proc.stdout.close()
        proc.stdin.close()
        return stdout, stderr

    def test_issue13183(self):
        script = """
            from bar import bar

            def foo():
                bar()

            def nope():
                pass

            def foobar():
                foo()
                nope()

            foobar()
        """
        commands = """
            from bar import bar
            break bar
            continue
            step
            step
            quit
        """
        bar = """
            def bar():
                pass
        """
        with open('bar.py', 'w') as f:
            f.write(textwrap.dedent(bar))
        self.addCleanup(test_support.unlink, 'bar.py')
        stdout, stderr = self.run_pdb(script, commands)
        self.assertTrue(
            any('main.py(5)foo()->None' in l for l in stdout.splitlines()),
            'Fail to step into the caller after a return')


class PdbTestInput(object):
    """Context manager that makes testing Pdb in doctests easier."""

    def __init__(self, input):
        self.input = input

    def __enter__(self):
        self.real_stdin = sys.stdin
        sys.stdin = _FakeInput(self.input)

    def __exit__(self, *exc):
        sys.stdin = self.real_stdin


def write(x):
    print x

def test_pdb_displayhook():
    """This tests the custom displayhook for pdb.

    >>> def test_function(foo, bar):
    ...     import pdb; pdb.Pdb().set_trace()
    ...     pass

    >>> with PdbTestInput([
    ...     'foo',
    ...     'bar',
    ...     'for i in range(5): write(i)',
    ...     'continue',
    ... ]):
    ...     test_function(1, None)
    > <doctest test.test_pdb.test_pdb_displayhook[0]>(3)test_function()
    -> pass
    (Pdb) foo
    1
    (Pdb) bar
    (Pdb) for i in range(5): write(i)
    0
    1
    2
    3
    4
    (Pdb) continue
    """

def test_pdb_breakpoint_commands():
    """Test basic commands related to breakpoints.

    >>> def test_function():
    ...     import pdb; pdb.Pdb().set_trace()
    ...     print(1)
    ...     print(2)
    ...     print(3)
    ...     print(4)

    First, need to clear bdb state that might be left over from previous tests.
    Otherwise, the new breakpoints might get assigned different numbers.

    >>> from bdb import Breakpoint
    >>> Breakpoint.next = 1
    >>> Breakpoint.bplist = {}
    >>> Breakpoint.bpbynumber = [None]

    Now test the breakpoint commands.  NORMALIZE_WHITESPACE is needed because
    the breakpoint list outputs a tab for the "stop only" and "ignore next"
    lines, which we don't want to put in here.

    >>> with PdbTestInput([  # doctest: +NORMALIZE_WHITESPACE
    ...     'break 3',
    ...     'disable 1',
    ...     'ignore 1 10',
    ...     'condition 1 1 < 2',
    ...     'break 4',
    ...     'break 4',
    ...     'break',
    ...     'clear 3',
    ...     'break',
    ...     'condition 1',
    ...     'enable 1',
    ...     'clear 1',
    ...     'commands 2',
    ...     'print 42',
    ...     'end',
    ...     'continue',  # will stop at breakpoint 2 (line 4)
    ...     'clear',     # clear all!
    ...     'y',
    ...     'tbreak 5',
    ...     'continue',  # will stop at temporary breakpoint
    ...     'break',     # make sure breakpoint is gone
    ...     'continue',
    ... ]):
    ...    test_function()
    > <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>(3)test_function()
    -> print(1)
    (Pdb) break 3
    Breakpoint 1 at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:3
    (Pdb) disable 1
    (Pdb) ignore 1 10
    Will ignore next 10 crossings of breakpoint 1.
    (Pdb) condition 1 1 < 2
    (Pdb) break 4
    Breakpoint 2 at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:4
    (Pdb) break 4
    Breakpoint 3 at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:4
    (Pdb) break
    Num Type         Disp Enb   Where
    1   breakpoint   keep no    at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:3
            stop only if 1 < 2
            ignore next 10 hits
    2   breakpoint   keep yes   at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:4
    3   breakpoint   keep yes   at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:4
    (Pdb) clear 3
    Deleted breakpoint 3
    (Pdb) break
    Num Type         Disp Enb   Where
    1   breakpoint   keep no    at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:3
            stop only if 1 < 2
            ignore next 10 hits
    2   breakpoint   keep yes   at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:4
    (Pdb) condition 1
    Breakpoint 1 is now unconditional.
    (Pdb) enable 1
    (Pdb) clear 1
    Deleted breakpoint 1
    (Pdb) commands 2
    (com) print 42
    (com) end
    (Pdb) continue
    1
    42
    > <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>(4)test_function()
    -> print(2)
    (Pdb) clear
    Clear all breaks? y
    (Pdb) tbreak 5
    Breakpoint 4 at <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>:5
    (Pdb) continue
    2
    Deleted breakpoint 4
    > <doctest test.test_pdb.test_pdb_breakpoint_commands[0]>(5)test_function()
    -> print(3)
    (Pdb) break
    (Pdb) continue
    3
    4
    """


def test_pdb_skip_modules():
    """This illustrates the simple case of module skipping.

    >>> def skip_module():
    ...     import string
    ...     import pdb; pdb.Pdb(skip=['string*']).set_trace()
    ...     string.lower('FOO')

    >>> with PdbTestInput([
    ...     'step',
    ...     'continue',
    ... ]):
    ...     skip_module()
    > <doctest test.test_pdb.test_pdb_skip_modules[0]>(4)skip_module()
    -> string.lower('FOO')
    (Pdb) step
    --Return--
    > <doctest test.test_pdb.test_pdb_skip_modules[0]>(4)skip_module()->None
    -> string.lower('FOO')
    (Pdb) continue
    """


# Module for testing skipping of module that makes a callback
mod = imp.new_module('module_to_skip')
exec 'def foo_pony(callback): x = 1; callback(); return None' in mod.__dict__


def test_pdb_skip_modules_with_callback():
    """This illustrates skipping of modules that call into other code.

    >>> def skip_module():
    ...     def callback():
    ...         return None
    ...     import pdb; pdb.Pdb(skip=['module_to_skip*']).set_trace()
    ...     mod.foo_pony(callback)

    >>> with PdbTestInput([
    ...     'step',
    ...     'step',
    ...     'step',
    ...     'step',
    ...     'step',
    ...     'continue',
    ... ]):
    ...     skip_module()
    ...     pass  # provides something to "step" to
    > <doctest test.test_pdb.test_pdb_skip_modules_with_callback[0]>(5)skip_module()
    -> mod.foo_pony(callback)
    (Pdb) step
    --Call--
    > <doctest test.test_pdb.test_pdb_skip_modules_with_callback[0]>(2)callback()
    -> def callback():
    (Pdb) step
    > <doctest test.test_pdb.test_pdb_skip_modules_with_callback[0]>(3)callback()
    -> return None
    (Pdb) step
    --Return--
    > <doctest test.test_pdb.test_pdb_skip_modules_with_callback[0]>(3)callback()->None
    -> return None
    (Pdb) step
    --Return--
    > <doctest test.test_pdb.test_pdb_skip_modules_with_callback[0]>(5)skip_module()->None
    -> mod.foo_pony(callback)
    (Pdb) step
    > <doctest test.test_pdb.test_pdb_skip_modules_with_callback[1]>(10)<module>()
    -> pass  # provides something to "step" to
    (Pdb) continue
    """


def test_pdb_continue_in_bottomframe():
    """Test that "continue" and "next" work properly in bottom frame (issue #5294).

    >>> def test_function():
    ...     import pdb, sys; inst = pdb.Pdb()
    ...     inst.set_trace()
    ...     inst.botframe = sys._getframe()  # hackery to get the right botframe
    ...     print(1)
    ...     print(2)
    ...     print(3)
    ...     print(4)

    First, need to clear bdb state that might be left over from previous tests.
    Otherwise, the new breakpoints might get assigned different numbers.

    >>> from bdb import Breakpoint
    >>> Breakpoint.next = 1
    >>> Breakpoint.bplist = {}
    >>> Breakpoint.bpbynumber = [None]

    >>> with PdbTestInput([
    ...     'next',
    ...     'break 7',
    ...     'continue',
    ...     'next',
    ...     'continue',
    ...     'continue',
    ... ]):
    ...    test_function()
    > <doctest test.test_pdb.test_pdb_continue_in_bottomframe[0]>(4)test_function()
    -> inst.botframe = sys._getframe()  # hackery to get the right botframe
    (Pdb) next
    > <doctest test.test_pdb.test_pdb_continue_in_bottomframe[0]>(5)test_function()
    -> print(1)
    (Pdb) break 7
    Breakpoint 1 at <doctest test.test_pdb.test_pdb_continue_in_bottomframe[0]>:7
    (Pdb) continue
    1
    2
    > <doctest test.test_pdb.test_pdb_continue_in_bottomframe[0]>(7)test_function()
    -> print(3)
    (Pdb) next
    3
    > <doctest test.test_pdb.test_pdb_continue_in_bottomframe[0]>(8)test_function()
    -> print(4)
    (Pdb) continue
    4
    """

class ModuleInitTester(unittest.TestCase):

    def test_filename_correct(self):
        """
        In issue 7750, it was found that if the filename has a sequence that
        resolves to an escape character in a Python string (such as \t), it
        will be treated as the escaped character.
        """
        # the test_fn must contain something like \t
        # on Windows, this will create 'test_mod.py' in the current directory.
        # on Unix, this will create '.\test_mod.py' in the current directory.
        test_fn = '.\\test_mod.py'
        code = 'print("testing pdb")'
        with open(test_fn, 'w') as f:
            f.write(code)
        self.addCleanup(os.remove, test_fn)
        cmd = [sys.executable, '-m', 'pdb', test_fn,]
        proc = subprocess.Popen(cmd,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            )
        stdout, stderr = proc.communicate('quit\n')
        self.assertIn(code, stdout, "pdb munged the filename")


def test_main():
    from test import test_pdb
    test_support.run_doctest(test_pdb, verbosity=True)
    test_support.run_unittest(
        PdbTestCase,
        ModuleInitTester)

if __name__ == '__main__':
    test_main()



# system configuration generated and used by the sysconfig module
build_time_vars = {'AC_APPLE_UNIVERSAL_BUILD': 0,
 'AIX_GENUINE_CPLUSPLUS': 0,
 'AR': 'ar',
 'ARFLAGS': 'rc',
 'ASDLGEN': './Parser/asdl_c.py',
 'ASDLGEN_FILES': './Parser/asdl.py ./Parser/asdl_c.py',
 'AST_ASDL': './Parser/Python.asdl',
 'AST_C': 'Python/Python-ast.c',
 'AST_C_DIR': 'Python',
 'AST_H': 'Include/Python-ast.h',
 'AST_H_DIR': 'Include',
 'ATHEOS_THREADS': 0,
 'BASECFLAGS': '',
 'BASEMODLIBS': '',
 'BEOS_THREADS': 0,
 'BINDIR': '/opt/python2.7/bin',
 'BINLIBDEST': '/opt/python2.7/lib/python2.7',
 'BLDLIBRARY': 'libpython2.7.a',
 'BLDSHARED': 'ld -b',
 'BUILDEXE': '',
 'BUILDPYTHON': 'python',
 'CC': 'cc -Ae',
 'CCSHARED': '+z',
 'CFLAGS': '-DNDEBUG -O',
 'CFLAGSFORSHARED': '',
 'CONFIGFILES': 'configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
 'CONFIG_ARGS': "'--prefix=/opt/python2.7'",
 'CONFINCLUDEDIR': '/opt/python2.7/include',
 'CONFINCLUDEPY': '/opt/python2.7/include/python2.7',
 'COREPYTHONPATH': ':plat-hp-ux11:lib-tk:lib-old',
 'CPPFLAGS': '-I. -IInclude -I./Include',
 'CXX': 'c++',
 'C_THREADS': 0,
 'DESTDIRS': '/opt/python2.7 /opt/python2.7/lib /opt/python2.7/lib/python2.7 /opt/python2.7/lib/python2.7/lib-dynload',
 'DESTLIB': '/opt/python2.7/lib/python2.7',
 'DESTPATH': '',
 'DESTSHARED': '/opt/python2.7/lib/python2.7/lib-dynload',
 'DIRMODE': 755,
 'DIST': 'README ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in Include Lib Misc Demo Ext-dummy',
 'DISTDIRS': 'Include Lib Misc Demo Ext-dummy',
 'DISTFILES': 'README ChangeLog configure configure.ac acconfig.h pyconfig.h.in Makefile.pre.in',
 'DLINCLDIR': '.',
 'DLLLIBRARY': '',
 'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
 'DYNLOADFILE': 'dynload_hpux.o',
 'ENABLE_IPV6': 1,
 'ENSUREPIP': 'no',
 'EXE': '',
 'EXEMODE': 755,
 'EXTRAMACHDEPPATH': '',
 'EXTRAPLATDIR': '',
 'EXTRATESTOPTS': '',
 'EXTRA_CFLAGS': '',
 'FILEMODE': 644,
 'FLOCK_NEEDS_LIBBSD': 0,
 'GETPGRP_HAVE_ARG': 0,
 'GETTIMEOFDAY_NO_TZ': 0,
 'GLHACK': '-Dclear=__GLclear',
 'GNULD': 'no',
 'GRAMMAR_C': 'Python/graminit.c',
 'GRAMMAR_H': 'Include/graminit.h',
 'GRAMMAR_INPUT': './Grammar/Grammar',
 'HAVE_ACOSH': 1,
 'HAVE_ADDRINFO': 1,
 'HAVE_ALARM': 1,
 'HAVE_ALLOCA_H': 1,
 'HAVE_ALTZONE': 0,
 'HAVE_ASINH': 1,
 'HAVE_ASM_TYPES_H': 0,
 'HAVE_ATANH': 1,
 'HAVE_ATTRIBUTE_FORMAT_PARSETUPLE': 0,
 'HAVE_BIND_TEXTDOMAIN_CODESET': 0,
 'HAVE_BLUETOOTH_BLUETOOTH_H': 0,
 'HAVE_BLUETOOTH_H': 0,
 'HAVE_BROKEN_NICE': 0,
 'HAVE_BROKEN_PIPE_BUF': 0,
 'HAVE_BROKEN_POLL': 0,
 'HAVE_BROKEN_POSIX_SEMAPHORES': 0,
 'HAVE_BROKEN_PTHREAD_SIGMASK': 0,
 'HAVE_BROKEN_SEM_GETVALUE': 0,
 'HAVE_BROKEN_UNSETENV': 0,
 'HAVE_C99_BOOL': 1,
 'HAVE_CHFLAGS': 0,
 'HAVE_CHOWN': 1,
 'HAVE_CHROOT': 0,
 'HAVE_CLOCK': 1,
 'HAVE_CONFSTR': 1,
 'HAVE_CONIO_H': 0,
 'HAVE_COPYSIGN': 1,
 'HAVE_CTERMID': 1,
 'HAVE_CTERMID_R': 0,
 'HAVE_CURSES_H': 1,
 'HAVE_CURSES_IS_TERM_RESIZED': 0,
 'HAVE_CURSES_RESIZETERM': 0,
 'HAVE_CURSES_RESIZE_TERM': 0,
 'HAVE_DECL_ISFINITE': 1,
 'HAVE_DECL_ISINF': 1,
 'HAVE_DECL_ISNAN': 1,
 'HAVE_DECL_TZNAME': 1,
 'HAVE_DEVICE_MACROS': 0,
 'HAVE_DEV_PTC': 0,
 'HAVE_DEV_PTMX': 1,
 'HAVE_DIRECT_H': 0,
 'HAVE_DIRENT_H': 1,
 'HAVE_DLFCN_H': 1,
 'HAVE_DLOPEN': 1,
 'HAVE_DUP2': 1,
 'HAVE_DYNAMIC_LOADING': 1,
 'HAVE_EPOLL': 0,
 'HAVE_ERF': 1,
 'HAVE_ERFC': 1,
 'HAVE_ERRNO_H': 1,
 'HAVE_EXECV': 1,
 'HAVE_EXPM1': 1,
 'HAVE_FCHDIR': 1,
 'HAVE_FCHMOD': 1,
 'HAVE_FCHOWN': 1,
 'HAVE_FCNTL_H': 1,
 'HAVE_FDATASYNC': 1,
 'HAVE_FINITE': 1,
 'HAVE_FLOCK': 1,
 'HAVE_FORK': 1,
 'HAVE_FORKPTY': 0,
 'HAVE_FPATHCONF': 1,
 'HAVE_FSEEK64': 0,
 'HAVE_FSEEKO': 1,
 'HAVE_FSTATVFS': 1,
 'HAVE_FSYNC': 1,
 'HAVE_FTELL64': 0,
 'HAVE_FTELLO': 1,
 'HAVE_FTIME': 1,
 'HAVE_FTRUNCATE': 1,
 'HAVE_GAI_STRERROR': 1,
 'HAVE_GAMMA': 1,
 'HAVE_GCC_ASM_FOR_X87': 0,
 'HAVE_GETADDRINFO': 1,
 'HAVE_GETCWD': 1,
 'HAVE_GETC_UNLOCKED': 1,
 'HAVE_GETGROUPS': 1,
 'HAVE_GETHOSTBYNAME': 1,
 'HAVE_GETHOSTBYNAME_R': 0,
 'HAVE_GETHOSTBYNAME_R_3_ARG': 0,
 'HAVE_GETHOSTBYNAME_R_5_ARG': 0,
 'HAVE_GETHOSTBYNAME_R_6_ARG': 0,
 'HAVE_GETITIMER': 1,
 'HAVE_GETLOADAVG': 0,
 'HAVE_GETLOGIN': 1,
 'HAVE_GETNAMEINFO': 1,
 'HAVE_GETPAGESIZE': 0,
 'HAVE_GETPEERNAME': 1,
 'HAVE_GETPGID': 1,
 'HAVE_GETPGRP': 1,
 'HAVE_GETPID': 1,
 'HAVE_GETPRIORITY': 1,
 'HAVE_GETPWENT': 1,
 'HAVE_GETRESGID': 1,
 'HAVE_GETRESUID': 1,
 'HAVE_GETSID': 1,
 'HAVE_GETSPENT': 0,
 'HAVE_GETSPNAM': 0,
 'HAVE_GETTIMEOFDAY': 1,
 'HAVE_GETWD': 1,
 'HAVE_GRP_H': 1,
 'HAVE_HSTRERROR': 0,
 'HAVE_HYPOT': 1,
 'HAVE_IEEEFP_H': 0,
 'HAVE_INET_ATON': 1,
 'HAVE_INET_PTON': 1,
 'HAVE_INITGROUPS': 1,
 'HAVE_INT32_T': 1,
 'HAVE_INT64_T': 1,
 'HAVE_INTTYPES_H': 1,
 'HAVE_IO_H': 0,
 'HAVE_KILL': 1,
 'HAVE_KILLPG': 1,
 'HAVE_KQUEUE': 0,
 'HAVE_LANGINFO_H': 1,
 'HAVE_LARGEFILE_SUPPORT': 1,
 'HAVE_LCHFLAGS': 0,
 'HAVE_LCHMOD': 1,
 'HAVE_LCHOWN': 1,
 'HAVE_LGAMMA': 1,
 'HAVE_LIBDL': 1,
 'HAVE_LIBDLD': 1,
 'HAVE_LIBIEEE': 0,
 'HAVE_LIBINTL_H': 0,
 'HAVE_LIBREADLINE': 0,
 'HAVE_LIBRESOLV': 0,
 'HAVE_LIBUTIL_H': 0,
 'HAVE_LINK': 1,
 'HAVE_LINUX_NETLINK_H': 0,
 'HAVE_LINUX_TIPC_H': 0,
 'HAVE_LOG1P': 1,
 'HAVE_LONG_DOUBLE': 1,
 'HAVE_LONG_LONG': 1,
 'HAVE_LSTAT': 1,
 'HAVE_MAKEDEV': 0,
 'HAVE_MEMMOVE': 1,
 'HAVE_MEMORY_H': 1,
 'HAVE_MKFIFO': 1,
 'HAVE_MKNOD': 1,
 'HAVE_MKTIME': 1,
 'HAVE_MMAP': 1,
 'HAVE_MREMAP': 0,
 'HAVE_NCURSES_H': 0,
 'HAVE_NDIR_H': 0,
 'HAVE_NETPACKET_PACKET_H': 0,
 'HAVE_NICE': 1,
 'HAVE_OPENPTY': 0,
 'HAVE_OSX105_SDK': 0,
 'HAVE_PATHCONF': 1,
 'HAVE_PAUSE': 1,
 'HAVE_PLOCK': 1,
 'HAVE_POLL': 1,
 'HAVE_POLL_H': 1,
 'HAVE_PROCESS_H': 0,
 'HAVE_PROTOTYPES': 1,
 'HAVE_PTH': 0,
 'HAVE_PTHREAD_ATFORK': 1,
 'HAVE_PTHREAD_DESTRUCTOR': 0,
 'HAVE_PTHREAD_H': 1,
 'HAVE_PTHREAD_INIT': 0,
 'HAVE_PTHREAD_SIGMASK': 1,
 'HAVE_PTY_H': 0,
 'HAVE_PUTENV': 1,
 'HAVE_READLINK': 1,
 'HAVE_REALPATH': 1,
 'HAVE_RL_CALLBACK': 0,
 'HAVE_RL_CATCH_SIGNAL': 0,
 'HAVE_RL_COMPLETION_APPEND_CHARACTER': 0,
 'HAVE_RL_COMPLETION_DISPLAY_MATCHES_HOOK': 0,
 'HAVE_RL_COMPLETION_MATCHES': 0,
 'HAVE_RL_COMPLETION_SUPPRESS_APPEND': 0,
 'HAVE_RL_PRE_INPUT_HOOK': 0,
 'HAVE_ROUND': 1,
 'HAVE_SELECT': 1,
 'HAVE_SEM_GETVALUE': 1,
 'HAVE_SEM_OPEN': 1,
 'HAVE_SEM_TIMEDWAIT': 0,
 'HAVE_SEM_UNLINK': 1,
 'HAVE_SETEGID': 1,
 'HAVE_SETEUID': 1,
 'HAVE_SETGID': 1,
 'HAVE_SETGROUPS': 1,
 'HAVE_SETITIMER': 1,
 'HAVE_SETLOCALE': 1,
 'HAVE_SETPGID': 1,
 'HAVE_SETPGRP': 1,
 'HAVE_SETREGID': 1,
 'HAVE_SETRESGID': 1,
 'HAVE_SETRESUID': 1,
 'HAVE_SETREUID': 1,
 'HAVE_SETSID': 1,
 'HAVE_SETUID': 1,
 'HAVE_SETVBUF': 1,
 'HAVE_SHADOW_H': 1,
 'HAVE_SIGACTION': 1,
 'HAVE_SIGINTERRUPT': 1,
 'HAVE_SIGNAL_H': 1,
 'HAVE_SIGRELSE': 1,
 'HAVE_SNPRINTF': 1,
 'HAVE_SOCKADDR_SA_LEN': 0,
 'HAVE_SOCKADDR_STORAGE': 1,
 'HAVE_SOCKETPAIR': 1,
 'HAVE_SPAWN_H': 0,
 'HAVE_SSIZE_T': 1,
 'HAVE_STATVFS': 1,
 'HAVE_STAT_TV_NSEC': 0,
 'HAVE_STAT_TV_NSEC2': 0,
 'HAVE_STDARG_PROTOTYPES': 1,
 'HAVE_STDINT_H': 1,
 'HAVE_STDLIB_H': 1,
 'HAVE_STRDUP': 1,
 'HAVE_STRFTIME': 1,
 'HAVE_STRINGS_H': 1,
 'HAVE_STRING_H': 1,
 'HAVE_STROPTS_H': 1,
 'HAVE_STRUCT_STAT_ST_BIRTHTIME': 0,
 'HAVE_STRUCT_STAT_ST_BLKSIZE': 1,
 'HAVE_STRUCT_STAT_ST_BLOCKS': 1,
 'HAVE_STRUCT_STAT_ST_FLAGS': 0,
 'HAVE_STRUCT_STAT_ST_GEN': 0,
 'HAVE_STRUCT_STAT_ST_RDEV': 1,
 'HAVE_STRUCT_TM_TM_ZONE': 0,
 'HAVE_ST_BLOCKS': 1,
 'HAVE_SYMLINK': 1,
 'HAVE_SYSCONF': 1,
 'HAVE_SYSEXITS_H': 1,
 'HAVE_SYS_AUDIOIO_H': 0,
 'HAVE_SYS_BSDTTY_H': 1,
 'HAVE_SYS_DIR_H': 0,
 'HAVE_SYS_EPOLL_H': 0,
 'HAVE_SYS_EVENT_H': 0,
 'HAVE_SYS_FILE_H': 1,
 'HAVE_SYS_LOADAVG_H': 0,
 'HAVE_SYS_LOCK_H': 1,
 'HAVE_SYS_MKDEV_H': 0,
 'HAVE_SYS_MODEM_H': 1,
 'HAVE_SYS_NDIR_H': 0,
 'HAVE_SYS_PARAM_H': 1,
 'HAVE_SYS_POLL_H': 1,
 'HAVE_SYS_RESOURCE_H': 1,
 'HAVE_SYS_SELECT_H': 1,
 'HAVE_SYS_SOCKET_H': 1,
 'HAVE_SYS_STATVFS_H': 1,
 'HAVE_SYS_STAT_H': 1,
 'HAVE_SYS_TERMIO_H': 1,
 'HAVE_SYS_TIMES_H': 1,
 'HAVE_SYS_TIME_H': 1,
 'HAVE_SYS_TYPES_H': 1,
 'HAVE_SYS_UN_H': 1,
 'HAVE_SYS_UTSNAME_H': 1,
 'HAVE_SYS_WAIT_H': 1,
 'HAVE_TCGETPGRP': 1,
 'HAVE_TCSETPGRP': 1,
 'HAVE_TEMPNAM': 1,
 'HAVE_TERMIOS_H': 1,
 'HAVE_TERM_H': 1,
 'HAVE_TGAMMA': 1,
 'HAVE_THREAD_H': 0,
 'HAVE_TIMEGM': 0,
 'HAVE_TIMES': 1,
 'HAVE_TMPFILE': 1,
 'HAVE_TMPNAM': 1,
 'HAVE_TMPNAM_R': 0,
 'HAVE_TM_ZONE': 0,
 'HAVE_TRUNCATE': 1,
 'HAVE_TZNAME': 1,
 'HAVE_UCS4_TCL': 0,
 'HAVE_UINT32_T': 1,
 'HAVE_UINT64_T': 1,
 'HAVE_UINTPTR_T': 1,
 'HAVE_UNAME': 1,
 'HAVE_UNISTD_H': 1,
 'HAVE_UNSETENV': 1,
 'HAVE_USABLE_WCHAR_T': 0,
 'HAVE_UTIL_H': 0,
 'HAVE_UTIMES': 1,
 'HAVE_UTIME_H': 1,
 'HAVE_WAIT3': 1,
 'HAVE_WAIT4': 1,
 'HAVE_WAITPID': 1,
 'HAVE_WCHAR_H': 1,
 'HAVE_WCSCOLL': 1,
 'HAVE_WORKING_TZSET': 1,
 'HAVE_ZLIB_COPY': 0,
 'HAVE__GETPTY': 0,
 'HGBRANCH': '',
 'HGTAG': '',
 'HGVERSION': '',
 'HOST_GNU_TYPE': 'ia64-hp-hpux11.31',
 'HURD_C_THREADS': 0,
 'INCLDIRSTOMAKE': '/opt/python2.7/include /opt/python2.7/include /opt/python2.7/include/python2.7 /opt/python2.7/include/python2.7',
 'INCLUDEDIR': '/opt/python2.7/include',
 'INCLUDEPY': '/opt/python2.7/include/python2.7',
 'INSTALL': './install-sh -c',
 'INSTALL_DATA': './install-sh -c -m 644',
 'INSTALL_PROGRAM': './install-sh -c',
 'INSTALL_SCRIPT': './install-sh -c',
 'INSTALL_SHARED': './install-sh -c -m 555',
 'INSTSONAME': 'libpython2.7.a',
 'LDCXXSHARED': 'ld -b',
 'LDFLAGS': '',
 'LDLAST': '',
 'LDLIBRARY': 'libpython2.7.a',
 'LDLIBRARYDIR': '',
 'LDSHARED': 'ld -b',
 'LIBC': '',
 'LIBDEST': '/opt/python2.7/lib/python2.7',
 'LIBDIR': '/opt/python2.7/lib',
 'LIBFFI_INCLUDEDIR': '',
 'LIBM': '-lm',
 'LIBOBJDIR': 'Python/',
 'LIBOBJS': '',
 'LIBP': '/opt/python2.7/lib/python2.7',
 'LIBPC': '/opt/python2.7/lib/pkgconfig',
 'LIBPL': '/opt/python2.7/lib/python2.7/config',
 'LIBRARY': 'libpython2.7.a',
 'LIBRARY_OBJS': '\\',
 'LIBS': '-lnsl -lrt -ldld -ldl  -lpthread',
 'LIBSUBDIRS': 'lib-tk lib-tk/test lib-tk/test/test_tkinter \\',
 'LINKCC': 'cc -Ae',
 'LINKFORSHARED': '-Wl,-E -Wl,+s',
 'LN': 'ln',
 'LOCALMODLIBS': '',
 'MACHDEP': 'hp-ux11',
 'MACHDEPPATH': ':plat-hp-ux11',
 'MACHDEPS': 'plat-hp-ux11',
 'MACHDEP_OBJS': '',
 'MACHDESTLIB': '/opt/python2.7/lib/python2.7',
 'MACH_C_THREADS': 0,
 'MACOSX_DEPLOYMENT_TARGET': '',
 'MAINCC': 'cc -Ae',
 'MAJOR_IN_MKDEV': 0,
 'MAJOR_IN_SYSMACROS': 1,
 'MAKESETUP': './Modules/makesetup',
 'MANDIR': '/opt/python2.7/share/man',
 'MEMTESTOPTS': '-l -x test_subprocess test_io test_lib2to3 \\ -x test_dl test___all__ test_fork1 \\',
 'MKDIR_P': './install-sh -c -d',
 'MODLIBS': '',
 'MODOBJS': 'Modules/threadmodule.o  Modules/signalmodule.o  Modules/posixmodule.o  Modules/errnomodule.o  Modules/pwdmodule.o  Modules/_sre.o  Modules/_codecsmodule.o  Modules/_weakref.o  Modules/zipimport.o  Modules/symtablemodule.o  Modules/xxsubtype.o',
 'MODULE_OBJS': '\\',
 'MULTIARCH': '',
 'MVWDELCH_IS_EXPRESSION': 0,
 'OBJECT_OBJS': '\\',
 'OLDPATH': ':lib-old',
 'OPT': '-DNDEBUG -O',
 'OTHER_LIBTOOL_OPT': '',
 'PACKAGE_BUGREPORT': 0,
 'PACKAGE_NAME': 0,
 'PACKAGE_STRING': 0,
 'PACKAGE_TARNAME': 0,
 'PACKAGE_URL': 0,
 'PACKAGE_VERSION': 0,
 'PARSER_HEADERS': '\\',
 'PARSER_OBJS': '\\ Parser/myreadline.o Parser/tokenizer.o',
 'PGEN': 'Parser/pgen',
 'PGENOBJS': '\\ \\',
 'PGENSRCS': '\\ \\',
 'PGOBJS': '\\',
 'PGSRCS': '\\',
 'PLATDIR': 'plat-hp-ux11',
 'PLATMACDIRS': 'plat-mac plat-mac/Carbon plat-mac/lib-scriptpackages \\',
 'PLATMACPATH': ':plat-mac:plat-mac/lib-scriptpackages',
 'POBJS': '\\',
 'POSIX_SEMAPHORES_NOT_ENABLED': 0,
 'PROFILE_TASK': './Tools/pybench/pybench.py -n 2 --with-gc --with-syscheck',
 'PSRCS': '\\',
 'PTHREAD_SYSTEM_SCHED_SUPPORTED': 1,
 'PURIFY': '',
 'PYLONG_BITS_IN_DIGIT': 0,
 'PYTHON': 'python',
 'PYTHONFRAMEWORK': '',
 'PYTHONFRAMEWORKDIR': 'no-framework',
 'PYTHONFRAMEWORKINSTALLDIR': '',
 'PYTHONFRAMEWORKPREFIX': '',
 'PYTHONPATH': ':plat-hp-ux11:lib-tk:lib-old',
 'PYTHON_FOR_BUILD': './python -E',
 'PYTHON_HEADERS': '\\',
 'PYTHON_OBJS': '\\',
 'PY_CFLAGS': '-DNDEBUG -O -I. -IInclude -I./Include  -DPy_BUILD_CORE',
 'PY_FORMAT_LONG_LONG': '"ll"',
 'PY_FORMAT_SIZE_T': '"z"',
 'PY_UNICODE_TYPE': 'unsigned short',
 'Py_DEBUG': 0,
 'Py_ENABLE_SHARED': 0,
 'Py_UNICODE_SIZE': 2,
 'Py_USING_UNICODE': 1,
 'QUICKTESTOPTS': '-l -x test_subprocess test_io test_lib2to3 \\',
 'RANLIB': 'ranlib',
 'RESSRCDIR': 'Mac/Resources/framework',
 'RETSIGTYPE': 'void',
 'RUNSHARED': '',
 'SCRIPTDIR': '/opt/python2.7/lib',
 'SETPGRP_HAVE_ARG': 0,
 'SGI_ABI': '',
 'SHELL': '/bin/sh',
 'SHLIBS': '-lnsl -lrt -ldld -ldl  -lpthread',
 'SHLIB_EXT': '".so"',
 'SIGNAL_OBJS': '',
 'SIGNED_RIGHT_SHIFT_ZERO_FILLS': 0,
 'SITEPATH': '',
 'SIZEOF_DOUBLE': 8,
 'SIZEOF_FLOAT': 4,
 'SIZEOF_FPOS_T': 8,
 'SIZEOF_INT': 4,
 'SIZEOF_LONG': 4,
 'SIZEOF_LONG_DOUBLE': 16,
 'SIZEOF_LONG_LONG': 8,
 'SIZEOF_OFF_T': 8,
 'SIZEOF_PID_T': 4,
 'SIZEOF_PTHREAD_T': 4,
 'SIZEOF_SHORT': 2,
 'SIZEOF_SIZE_T': 4,
 'SIZEOF_TIME_T': 4,
 'SIZEOF_UINTPTR_T': 4,
 'SIZEOF_VOID_P': 4,
 'SIZEOF_WCHAR_T': 4,
 'SIZEOF__BOOL': 1,
 'SO': '.so',
 'SRCDIRS': 'Parser Grammar Objects Python Modules Mac',
 'SRC_GDB_HOOKS': './Tools/gdb/libpython.py',
 'STDC_HEADERS': 1,
 'STRICT_SYSV_CURSES': "/* Don't use ncurses extensions */",
 'STRINGLIB_HEADERS': '\\',
 'SUBDIRS': '',
 'SUBDIRSTOO': 'Include Lib Misc Demo',
 'SVNVERSION': 'echo Unversioned directory',
 'SYSLIBS': '-lm',
 'SYS_SELECT_WITH_SYS_TIME': 1,
 'TANH_PRESERVES_ZERO_SIGN': 1,
 'TCLTK_INCLUDES': '',
 'TCLTK_LIBS': '',
 'TESTOPTS': '-l',
 'TESTPATH': '',
 'TESTPROG': './Lib/test/regrtest.py',
 'TESTPYTHON': './python -Wd -3 -E -tt',
 'TESTPYTHONOPTS': '',
 'THREADOBJ': 'Python/thread.o',
 'TIME_WITH_SYS_TIME': 1,
 'TKPATH': ':lib-tk',
 'TM_IN_SYS_TIME': 0,
 'UNICODE_OBJS': 'Objects/unicodeobject.o Objects/unicodectype.o',
 'UNIVERSALSDK': '',
 'USE_TOOLBOX_OBJECT_GLUE': 0,
 'VA_LIST_IS_ARRAY': 0,
 'VERSION': '2.7',
 'WANT_SIGFPE_HANDLER': 0,
 'WANT_WCTYPE_FUNCTIONS': 0,
 'WINDOW_HAS_FLAGS': 0,
 'WITH_DOC_STRINGS': 1,
 'WITH_DYLD': 0,
 'WITH_LIBINTL': 0,
 'WITH_NEXT_FRAMEWORK': 0,
 'WITH_PYMALLOC': 1,
 'WITH_THREAD': 1,
 'WITH_TSC': 0,
 'WITH_VALGRIND': 0,
 'X87_DOUBLE_ROUNDING': 0,
 'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
 'abs_builddir': '/opt/yjh/Python-2.7.9',
 'abs_srcdir': '/opt/yjh/Python-2.7.9',
 'build': 'ia64-hp-hpux11.31',
 'datarootdir': '/opt/python2.7/share',
 'exec_prefix': '/opt/python2.7',
 'host': 'ia64-hp-hpux11.31',
 'prefix': '/opt/python2.7',
 'srcdir': '.'}



import platform
import os

_getcpumemory = None
_getcpunum = None
_getcpu = None
_getmemory = None
_getswapmemory = None
_getcpustat = None
_getcpucore = None
_getcputype = None
_getmemorysize = None
_getdisk = None
_getalldisk = None
_getdiskio = None
_initzfsinfo = None
_getzfsinfo = None
_getzpoolperfinfo = None
_initDisk = None
_endDisk = None
_getnet = None
_getallnet = None
_getproc = None
_getallproc = None
_gettopnandgroupproc = None
_getlogevent = None
_initlogevent = None
_getfilecheckevents = None
_initfilecheckevent = None
_getmachinearc = None
_getServerInfoV2 = None
_initSysLogRedirect = None
_getnicspeed = None
_getnetstatcounters = None
_getntpinfo = None
_getnicinfolist = None

processes = []
system = platform.system().lower()

class _DiskPerf:
    UNSUPPORTED = -1

    def __init__(self):
        self.mountPoint = None
        self.deviceId = None
        self.fileSystem = None
        self.totalSpace = 0
        self.usedSpace = 0
        self.usedPercent = 0.0
        self.freeSpace = 0
        self.freePercent = 0.0
        self.blksize = 0
        self.readIops = 0
        self.writeIops = 0
        self.totalIops = 0
        self.readBps = 0
        self.writeBps = 0
        self.totalBps = 0
        self.readTime = _DiskPerf.UNSUPPORTED
        self.writeTime = _DiskPerf.UNSUPPORTED
        self.totalBusyTime = _DiskPerf.UNSUPPORTED
        self.totalWaitTime = _DiskPerf.UNSUPPORTED
        self.ioPercent = 0
        self.queueLength = 0
        self.inodePercent = 0
        self.busyMax = None
        self.diskName = None

    def __str__(self):
        buf = StringIO()
        buf.write('mountPoint=')
        buf.write(self.mountPoint) 
        buf.write('\ndeviceId=')
        buf.write(self.deviceId)
        buf.write('\nfileSystem=')
        buf.write(self.fileSystem)
        buf.write('\ntotalSpace=')
        buf.write(self.totalSpace)
        buf.write('\nusedSpace=')
        buf.write(self.usedSpace)
        buf.write('\nusedPercent=')
        buf.write(self.usedPercent)
        buf.write('\nfreeSpace=')
        buf.write(self.freeSpace)
        buf.write('\nfreePercent=')
        buf.write(self.freePercent)
        buf.write('\nself.blksize=')
        buf.write(self.blksize)
        buf.write('\nreadIops=')
        buf.write(self.readIops)
        buf.write('\nwriteIops=')
        buf.write(self.writeIops)
        buf.write('\nreadBps=')
        buf.write(self.readBps)
        buf.write('\nwriteBps=')
        buf.write(self.writeBps)
        buf.write('\nioPercent=')
        buf.write(self.ioPercent)
        buf.write('\nqueueLength=')
        buf.write(self.queueLength)

        return buf.getvalue()


class CPUStat:
    # -1 unsuppoerted
    UNSUPPORTED = -1

    def __init__(self):
        self.runqueue = 0
        self.blockqueue = 0
        self.pendingqueue = 0
        self.waitqueue = 0
        self.forkCount = 0
        self.execCount = 0
        self.ctx = 0
        self.interrupt = 0
        self.syscall = 0


if 'linux' == system:
    import cpu_memory as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory

    import system_info_linux as system_info
    _getcpunum = system_info.GetCPUNum
    _getcputype = system_info.GetCPUType
    _getmemorysize = system_info.GetMemorySize
    _getOSRelease = system_info.GetOSRelease
    import disk_linux
    _getdisk = disk_linux.getDisk
    import network_linux
    _getnet = network_linux.GetNicUtil
    import proc_linux
    _getproc = proc_linux.GetProc
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    import file_check_event
    _getfilecheckevents = file_check_event.get_file_check_events
    _initfilecheckevent = file_check_event.init_file_check_watcher
    _getmachinearc = system_info.GetMachineArch
elif 'sunos' == system:
    import cpu_memory_sunos as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory
    _getcpu = cpu_memory.GetCpu
    _getmemory = cpu_memory.GetMemory
    _getswapmemory = cpu_memory.GetSwapMemory
    _getcpustat = cpu_memory.GetCpuStat
    _getcpucore = cpu_memory.GetCpuCore
    import system_info_sunos as system_info
    import system_info_sunos
    _getcpunum = system_info_sunos.GetCPUNum
    _getcputype = system_info_sunos.GetCPUType
    _getmemorysize = system_info_sunos.GetMemorySize
    _getOSRelease = system_info_sunos.GetOSRelease
    import disk_sunos 
    _getdisk = disk_sunos.getDisk #fs
    _getalldisk = disk_sunos.GetDiskTotalIO
    _getdiskio = disk_sunos.GetDiskIO
    _getzfsinfo = disk_sunos.GetZfsInfo
    _initzfsinfo = disk_sunos.initUpdateZfs
    _getzpoolperfinfo = disk_sunos.GetZpoolPerfInfo
    import network_sunos
    _getnet = network_sunos.GetNicUtil
    _getallnet = network_sunos.GetNetInfo
    _getnicspeed = network_sunos.GetNicLinkSpeed
    import netstat_sunos
    _getnetstatcounters = netstat_sunos.GetNetstatCounters
    _getntpinfo = system_info_sunos.GetNtpInfo
    import nic_info_sunos
    _getnicinfolist = nic_info_sunos.GetNicInfoList
    import proc_sunos
    _getproc = proc_sunos.GetProc
    _getallproc = proc_sunos.GetTotalProc
    _gettopnandgroupproc = proc_sunos.GetTopNAndGroupProc
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    import file_check_event
    _getfilecheckevents = file_check_event.get_file_check_events
    _initfilecheckevent = file_check_event.init_file_check_watcher
    _getmachinearc = system_info_sunos.GetMachineArch
    _getServerInfoV2 = system_info_sunos.GetServerInfoV2
elif 'aix' == system: 
    os.environ['LANG'] = 'en_US'
    import cpu_memory_aix as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory
    _getcpu = cpu_memory.GetCpu
    _getmemory = cpu_memory.GetMemory
    _getswapmemory = cpu_memory.GetSwapMemory
    _getcpustat = cpu_memory.GetCpuStat
    _getcpucore = cpu_memory.GetCpuCore
    import system_info_aix as system_info
    import system_info_aix 
    _getcpunum = system_info_aix.GetCPUNum
    _getcputype = system_info_aix.GetCPUType
    _getmemorysize = system_info_aix.GetMemorySize
    _getOSRelease = system_info_aix.GetOSRelease
    _initSysLogRedirect = system_info_aix.InitErrptRedirect
    import disk_aix 
    _getdisk = disk_aix.getDisk
    _getalldisk = disk_aix.GetDiskTotalIO
    _getdiskio = disk_aix.GetDiskIO
    import network_aix
    _getnet = network_aix.GetNicUtil
    _getallnet = network_aix.GetNetInfo
    _getnicspeed = network_aix.GetNicLinkSpeed
    import netstat_aix
    _getnetstatcounters = netstat_aix.GetNetstatCounters
    _getntpinfo = system_info_aix.GetNtpInfo
    import nic_info_aix
    _getnicinfolist = nic_info_aix.GetNicInfoList
    import proc_aix
    _getproc = proc_aix.GetProc
    _getallproc = proc_aix.GetTotalProc
    _gettopnandgroupproc = proc_aix.GetTopNAndGroupProc
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    import file_check_event
    _getfilecheckevents = file_check_event.get_file_check_events
    _initfilecheckevent = file_check_event.init_file_check_watcher
    _getmachinearc = system_info_aix.GetMachineArch
    _getServerInfoV2 = system_info_aix.GetServerInfoV2
elif 'hp-ux' == system:
    import cpu_memory_hpux as cpu_memory
    _getcpumemory = cpu_memory.GetCpuMemory
    _getcpu = cpu_memory.GetCpu
    _getmemory = cpu_memory.GetMemory
    _getswapmemory = cpu_memory.GetSwapMemory
    _getcpustat = cpu_memory.GetCpuStat
    _getcpucore = cpu_memory.GetCpuCore

    import system_info_hpux as system_info
    _getcpunum = system_info.GetCPUNum
    _getcputype = system_info.GetCPUType
    _getmemorysize = system_info.GetMemorySize
    _getOSRelease = system_info.GetOSRelease
    _getmachinearc = system_info.GetMachineArch
    import disk_hpux
    _getdisk = disk_hpux.getDisk
    _getalldisk = disk_hpux.GetDiskTotalIO
    _getdiskio = disk_hpux.GetDiskIO
    import proc_hpux
    _getproc = proc_hpux.GetProc
    _getallproc = proc_hpux.GetTotalProc
    _gettopnandgroupproc = proc_hpux.GetTopNAndGroupProc
    import network_hpux
    _getnet = network_hpux.GetNicUtil
    _getallnet = network_hpux.GetNetInfo
    _getnicspeed = network_hpux.GetNicLinkSpeed
    import netstat_hpux
    _getnetstatcounters = netstat_hpux.GetNetstatCounters
    _getntpinfo = system_info_hpux.GetNtpInfo
    import nic_info_hpux
    _getnicinfolist = nic_info_hpux.GetNicInfoList
    import log_event_linux
    _getlogevent = log_event_linux.getNewLogEvents
    _initlogevent = log_event_linux.initFileLogWatcher
    import file_check_event
    _getfilecheckevents = file_check_event.get_file_check_events
    _initfilecheckevent = file_check_event.init_file_check_watcher
    _getServerInfoV2 = system_info_hpux.GetServerInfoV2

def GetCpu():
    if _getcpu == None:
        return None
    return _getcpu()

def GetMemory():
    if _getmemory == None:
        return None
    return _getmemory()

def GetSwapMemory():
    if _getswapmemory == None:
        return None
    return _getswapmemory()

def GetCpuStat():
    if _getcpustat == None:
        return None
    return _getcpustat()

def GetCpuCore():
    if _getcpucore == None:
        return None
    return _getcpucore()

def GetCpuMemory():
    if _getcpumemory == None:
        return None
    return _getcpumemory()

def GetCPUNum():
    if _getcpunum == None:
        return None
    return _getcpunum()

def GetCPUType():
    if _getcputype == None:
        return None
    return _getcputype()

def GetMemorySize():
    if _getmemorysize == None:
        return None
    return _getmemorysize()

def GetUptime():
    import time
    return int(time.time())

def GetDisk():
    if _getdisk == None:
        return None
    return _getdisk()

def GetTotalDisk():
    if _getalldisk == None:
        return None
    return _getalldisk()

def GetDiskIO():
    if _getdiskio == None:
        return None
    return _getdiskio()

def InitZfsInfo():
    if _initzfsinfo == None:
        return None
    return _initzfsinfo()

def GetZfsInfo():
    if _getzfsinfo == None:
        return None
    return _getzfsinfo()

def GetZpoolPerfInfo():
    if _getzpoolperfinfo == None:
        return None
    return _getzpoolperfinfo()

def GetNicUtil():
    if _getnet == None:
        return None
    return _getnet()

def GetTotalNet():
    if _getallnet == None:
        return None
    return _getallnet()

def GetProc():
    if _getproc == None:
        return None
    return _getproc()

def GetTotalProc():
    if _getallproc == None:
        return None, None, None
    return _getallproc()

def GetTopNAndGroupProc(n):
    if _gettopnandgroupproc == None:
        return None, None
    return _gettopnandgroupproc(n)

def GetLogEvents():
    if _getlogevent == None:
        return None
    return _getlogevent()

def InitLogEvent():
    if _initlogevent == None:
        return None
    _initlogevent()

def GetFileCheckEvents():
    if _getfilecheckevents == None:
        return None
    return _getfilecheckevents()

def InitFileCheckEvent():
    if _initfilecheckevent == None:
        return None
    _initfilecheckevent()

def GetMachineArch():
    if _getmachinearc == None:
        return None
    return _getmachinearc()

import tcp_check

def TCPCheckSchedule(checkurl, timeout, onPerf):
    return tcp_check.tcpCheckSchedule(checkurl, timeout, onPerf)

def GetOSRelease():
    if _getOSRelease == None:
        return None
    return _getOSRelease()

def GetServerInfoV2():
    if _getServerInfoV2 == None:
        return None
    return _getServerInfoV2()

def GetNicLinkSpeed(iface):
    if _getnicspeed == None:
        return None
    return _getnicspeed(iface)

def GetNetstatCounters():
    if _getnetstatcounters == None:
        return None
    return _getnetstatcounters()

def GetNtpInfo():
    if _getntpinfo == None:
        return None
    return _getntpinfo()

def GetNicInfoList():
    if _getnicinfolist == None:
        return None
    return _getnicinfolist()

def GetLCore():
    lcore = 0
    try:
        lcore = int(GetServerInfoV2()["lCores"])
    except ValueError, e:
        lcore = 0

    return lcore

def InitSyslogRedirect():
    if _initSysLogRedirect == None:
        return
    _initSysLogRedirect()
    



import os, time
import threading
from StringIO import StringIO
import subprocess, shlex
import whatap.util.logging_util as logging_util
from whatap.agent.conf.configure import GetConfig
import whatap.util.thread_util as thread_util
import re
import psutil

from . import _DiskPerf

filesystems = { "btrfs": True, "ext2": True, "ext3": True, "ext4": True, "reiser": True, "xfs": True,
        "ffs": True, "ufs": True, "jfs": True, "jfs2": True, "vxfs": True, "hfs": True, "ntfs": True, "fat32": True,
        "zfs": True, "refs": True, "nfs": True, "nfs2": True, "nfs3": True, "nfs4": True, "cifs": True, "ocfs2": True }

zfsInfoCheck = False
_zfsUpdateStarted = False

class _PoolInfo:
    def __init__(self, name):
        self.name = name
        self.totalSpace = None
        self.diskList = []
        self.ioInfo = _IoInfo()
        self.oldTimeStamp = 0
        self.readIops = 0
        self.writeIops = 0
        self.readBps = 0
        self.writeBps = 0
        self.ioPercent = 0


    def setTotalSpace(self, totalSpace):
        self.totalSpace = totalSpace

    def setDisk(self, disk):
        self.diskList.append(disk)

    def initIoInfo(self):
        self.ioInfo.writeIoByte = 0
        self.ioInfo.writeIoCount = 0 
        self.ioInfo.readIoByte = 0
        self.ioInfo.readIoCount = 0 
        self.ioInfo.ioMillis = 0 

    def sumIoStats(self, newInfo):
        self.ioInfo.writeIoByte += newInfo.writeIoByte
        self.ioInfo.writeIoCount += newInfo.writeIoCount
        self.ioInfo.readIoByte += newInfo.readIoByte
        self.ioInfo.readIoCount += newInfo.readIoCount
        if self.ioInfo.ioMillis < newInfo.ioMillis:
            self.ioInfo.ioMillis = newInfo.ioMillis

    def calcIoStat(self, diffTime):
        self.writeBps = float(self.ioInfo.writeIoByte) / float(diffTime)
        self.writeIops = float(self.ioInfo.writeIoCount) / float(diffTime)
        self.readBps = float(self.ioInfo.readIoByte) / float(diffTime)
        self.readIops = float(self.ioInfo.readIoCount) / float(diffTime)
        self.ioPercent = float(self.ioInfo.ioMillis) / float(diffTime * 1000) * 100
        if self.ioPercent > float(100):
            self.ioPercent = float(100)
        self.initIoInfo()

            
 
class _DataSetInfo:
    def __init__(self, name):
        self.name = name
        self.pool = None
        self.used = 0
        self.avail = 0
        self.refer = 0
        self.quota = None
        self.refquota = None
        self.recordSize = 0

    def setDataSet(self, pool, used, avail, refer, quota, refquota, recordSize):
        self.pool = pool
        self.used = used
        self.avail = avail
        self.refer = refer
        self.quota = quota if quota != "none" and quota != "0" else None
        self.refquota = refquota if refquota != "none" and refquota != "0" else None
        self.recordSize = recordSize

    def setRecodeSize(self, recordSize):
        self.recordSize = recordSize


class _IoInfo:
    def __init__(self):
        self.writeIoByte= 0
        self.writeIoCount = 0
        self.readIoByte= 0
        self.readIoCount = 0
        self.readTime = 0
        self.writeTime = 0
        self.ioMillis = 0
        self.timestamp = 0

    def setIoInfo(self, writeIoByte, writeIoCount, readIoByte, readIoCount, ioMillis, timestamp):
        self.writeIoByte= writeIoByte
        self.writeIoCount = writeIoCount
        self.readIoByte= readIoByte
        self.readIoCount = readIoCount
        self.ioMillis = ioMillis
        self.timestamp = timestamp

def getDiskList():
    diskList = list(set(os.listdir("/dev/rdsk") + os.listdir("/dev/dsk")))
    return diskList

oldZfsDiskList = None
diskToPoolMap = None

def getZpoolInfo():
    global oldZfsDiskList
    diskList = getDiskList()

    if diskList == oldZfsDiskList:
        return None

    command = "zpool list -H -o name,size"
    try:
        process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        text = process.stdout.read()

        info_map = {}
        for line in text.strip('\n').split('\n'):
            name, size = line.split('\t')
            pool = _PoolInfo(name)
            pool.setTotalSpace(size)
            info_map[name] = pool

        info_map = getZpoolStatus(info_map, diskList)
        oldZfsDiskList = diskList
        return info_map
    except subprocess.CalledProcessError as e:
        logging_util.error("zpool list error  : {} - {}".format(type(e).__name__, e))
        return None
    except Exception as e:
        logging_util.error("zpool list error  : {} - {}".format(type(e).__name__, e))
        return None


globalDiskMap = None
def getDiskMap():
    global globalDiskMap
    if globalDiskMap:
        return globalDiskMap
    diskMap = {}
    #command = "paste -d, <(iostat -x | awk 'NR>2{print $1}') <(iostat -nx | awk 'NR>2{print $11}')"
    command = "tmp1=/tmp/tmpfile1 tmp2=/tmp/tmpfile2 && iostat -x | awk 'NR>2{print $1}' > $tmp1 && iostat -nx | awk 'NR>2{print $11}' > $tmp2 && paste -d, $tmp1 $tmp2 && rm $tmp1 $tmp2"
    try:
        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        text = process.stdout.read()
        for line in text.strip('\n').split('\n'):
            deviceName, deviceRealName = line.split(',')
            diskMap[deviceRealName] = deviceName
    except subprocess.CalledProcessError as e:
        logging_util.error("iostat error  : {} - {}".format(type(e).__name__, e))
        return None
    except Exception as e:
        logging_util.error("iostat error  : {} - {}".format(type(e).__name__, e))
        return None



    globalDiskMap = diskMap
    return diskMap

def getZpoolStatus(info_map, diskList):
    poolMap = {}
    global diskToPoolMap

    diskMap = getDiskMap()

    command = "zpool status"
    try:
        process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        text = process.stdout.read().strip('\n')
        poolKey = ""
        
        for line in re.split(r'[\n\t]+', text):
            status = [value for value in line.split(' ') if value]
            if status[0] in info_map.keys():
                poolKey = status[0]
            elif status[0] in diskList and poolKey in info_map:
                if poolKey == "": continue
                else: 
                    diskName = re.sub(r's\d+', '', status[0])
                    if diskName in diskMap:
                        info_map[poolKey].setDisk(diskName)
                        poolMap[diskMap[diskName]] = poolKey
        diskToPoolMap = poolMap
        return info_map
    except subprocess.CalledProcessError as e:
        logging_util.error("zpool status error  : {} - {}".format(type(e).__name__, e))
        return None
    except Exception as e:
        logging_util.error("zpool status error  : {} - {}".format(type(e).__name__, e))
        return None



def diffIoInfo(total, old):
    if total == None or old == None:
        return None
    ret = _IoInfo()
    ret.setIoInfo(total.writeIoByte - old.writeIoByte, total.writeIoCount- old.writeIoCount, total.readIoByte - old.readIoByte, total.readIoCount - old.readIoCount, total.ioMillis - old.ioMillis, total.timestamp - old.timestamp)
    ret.readTime = total.readTime - old.readTime
    ret.writeTime = total.writeTime - old.writeTime
    return ret

oldIOTotal = None
def getDiskIoInfo():
    ioStats = psutil.disk_io_counters()
    info = _IoInfo()
    now = time.time() 

    info.setIoInfo(ioStats.write_bytes, ioStats.write_count, ioStats.read_bytes, ioStats.read_count, ioStats.write_time + ioStats.read_time, now)

    global oldIOTotal
    if not oldIOTotal:
        oldIOTotal = info
        return

    diffIo = diffIoInfo(info, oldIOTotal)
    
    ret = _DiskPerf()
    diffTime = diffIo.timestamp
    ret.readBps = float(diffIo.readIoByte / diffTime)
    ret.writeBps = float(diffIo.writeIoByte / diffTime)
    ret.readIops = float(diffIo.readIoCount / diffTime)
    ret.writeIops = float(diffIo.writeIoCount / diffTime)

    oldIOTotal = info
    return ret

def getTotalDisk():
    disk = getDiskIoInfo()

#infra - combined: aggregate IO + busyMax/diskName in one perdisk call
oldIOTotalPerDisk = None
def GetDiskTotalIO():
    ioStats = psutil.disk_io_counters(perdisk=True)
    now = time.time()

    global oldIOTotalPerDisk
    if not oldIOTotalPerDisk:
        oldIOTotalPerDisk = (ioStats, now)
        return None

    oldStats, oldTime = oldIOTotalPerDisk
    diffTime = float(now - oldTime)
    if diffTime <= 0:
        return None

    total_read_bytes = 0
    total_write_bytes = 0
    total_read_count = 0
    total_write_count = 0
    max_busy = 0.0
    max_disk_name = ""

    for k, v in ioStats.items():
        if k not in oldStats:
            continue
        oldv = oldStats[k]
        total_read_bytes += v.read_bytes - oldv.read_bytes
        total_write_bytes += v.write_bytes - oldv.write_bytes
        total_read_count += v.read_count - oldv.read_count
        total_write_count += v.write_count - oldv.write_count

        busy_diff = float((v.read_time + v.write_time) - (oldv.read_time + oldv.write_time))
        busy_pct = 100.0 * busy_diff / (diffTime * 1000)
        if busy_pct > max_busy:
            max_busy = busy_pct
            max_disk_name = k

    ret = _DiskPerf()
    ret.readBps   = float(total_read_bytes) / diffTime
    ret.writeBps  = float(total_write_bytes) / diffTime
    ret.readIops  = float(total_read_count) / diffTime
    ret.writeIops = float(total_write_count) / diffTime
    ret.busyMax   = min(max_busy, 100.0)
    ret.diskName  = max_disk_name

    oldIOTotalPerDisk = (ioStats, now)
    return ret

oldIoInfoMapForInfra = None
def GetDiskIO():
    ioStats = psutil.disk_io_counters(perdisk=True)
    ioInfoMap = {}
    now = time.time() 

    for k, v in ioStats.items():
        info = _IoInfo()
        info.setIoInfo(v.write_bytes, v.write_count, v.read_bytes, v.read_count, v.write_time+ v.read_time, now)
        info.readTime = v.read_time
        info.writeTime = v.write_time
        ioInfoMap[k] = info

    global oldIoInfoMapForInfra
    if not oldIoInfoMapForInfra:
        oldIoInfoMapForInfra = ioInfoMap
        return None
    
    retArr = []
    for k, v in ioInfoMap.items():
        diffIo = diffIoInfo(v, oldIoInfoMapForInfra[k])
        if not diffIo:
            continue

        diffTime = diffIo.timestamp
        ret = _DiskPerf()
        ret.deviceId = k
        ret.readBps = float(diffIo.readIoByte) / float(diffTime)
        ret.writeBps = float(diffIo.writeIoByte) / float(diffTime)
        ret.readIops = float(diffIo.readIoCount) / float(diffTime)
        ret.writeIops = float(diffIo.writeIoCount) / float(diffTime)

        if diffIo.readIoCount == 0:
            ret.readTime = 0
        else:
            ret.readTime = float(diffIo.readTime) / float(diffIo.readIoCount)

        if diffIo.writeIoCount == 0:
            ret.writeTime = 0
        else:
            ret.writeTime = float(diffIo.writeTime) / float(diffIo.writeIoCount)

        ret.ioPercent = float(diffIo.ioMillis) / float(diffTime * 1000) * 100
        if ret.ioPercent > float(100):
            ret.ioPercent = float(100)

        retArr.append(ret)

    oldIoInfoMapForInfra = ioInfoMap
    return retArr
         

#server
oldIoInfoMap = None
def getDiskIoInfoPerDisk(diff):
    ioStats = psutil.disk_io_counters(perdisk=True)
    ioInfoMap = {}
    now = time.time() 

    for k, v in ioStats.items():
        info  = _IoInfo()
        info.setIoInfo(v.write_bytes, v.write_count, v.read_bytes, v.read_count, v.write_time+ v.read_time, now)
        ioInfoMap[k] = info

    if zpoolInfoMap == None:
        return 

    global oldIoInfoMap 
    if not oldIoInfoMap:
        oldIoInfoMap = ioInfoMap
        return

    for k, v in ioInfoMap.items():
        diffIo = diffIoInfo(v, oldIoInfoMap[k])
        if not diffIo:
            continue
        try:
            zpoolInfoMap[diskToPoolMap[k]].sumIoStats(diffIo)
        except KeyError, e:
            logging_util.debug("disk to pool error  : {} - {}".format(type(e).__name__, e))
            continue
        except TypeError, e:
            logging_util.debug("disk to pool error  : {} - {}".format(type(e).__name__, e))
            continue

    oldIoInfoMap = ioInfoMap
    for k, v in zpoolInfoMap.items():
        v.calcIoStat(diff)



def getZfsInfo():
    try:
        command = "zfs list -H -o name,used,avail,refer,quota,refquota,recordsize"
        process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        text = process.stdout.read()

        info_map = {}
        for line in text.strip('\n').split('\n'):
            name, used, avail, refer, quota, refquota, recordSize = line.split('\t')
            pool = name.split('/')[0]
            dataSet = _DataSetInfo(name)
            dataSet.setDataSet(pool, used, avail, refer, quota, refquota, recordSize)
            info_map[name] = dataSet
        return info_map
    except subprocess.CalledProcessError as e:
        logging_util.error("zfs list error  : {} - {}".format(type(e).__name__, e))
        return None
    except Exception as e:
        logging_util.error("zfs list error  : {} - {}".format(type(e).__name__, e))
        return None




zpoolInfoMap = None 
zfsInfoMap = None
oldTime = time.time()

def setZfsInfo():
    global oldTime
    now = time.time()

    global zpoolInfoMap 
    poolCheck = getZpoolInfo()
    if poolCheck != None:
        zpoolInfoMap = poolCheck

    global zfsInfoMap 
    zfsInfoMap = getZfsInfo()
    diffTime = now - oldTime
    getDiskIoInfoPerDisk(diffTime)

    oldTime = now

def parse_byte_value(byte_str):
    if byte_str.endswith('K'):
        return int(float(byte_str[:-1]) * 1024)
    elif byte_str.endswith('M'):
        return int(float(byte_str[:-1]) * 1024 * 1024)
    elif byte_str.endswith('G'):
        return int(float(byte_str[:-1]) * 1024 * 1024 * 1024)
    elif byte_str.endswith('T'):
        return int(float(byte_str[:-1]) * 1024 * 1024 * 1024 * 1024)
    else:
        return int(byte_str)

def parse_count_value(count_str):
    if count_str.endswith('K'):
        return int(float(count_str[:-1]) * 1000)
    elif count_str.endswith('M'):
        return int(float(count_str[:-1]) * 1000 * 1000)
    elif count_str.endswith('G'):
        return int(float(count_str[:-1]) * 1000 * 1000 * 1000)
    elif count_str.endswith('T'):
        return int(float(count_str[:-1]) * 1000 * 1000 * 1000 * 1000)
    else:
        return int(count_str)



def getAncestorQuota(dataSetName, infoMap):
    parts = dataSetName.split('/')
    for i in range(len(parts) - 1, 0, -1):
        ancestor_name = '/'.join(parts[:i])
        if ancestor_name in infoMap:
            ancestor = infoMap[ancestor_name]
            if ancestor.quota:
                return ancestor.quota
    return None

def getZFSDisk(diskPerf):
    if zfsInfoMap is None or zpoolInfoMap is None:
        return diskPerf

    dataSetInfo = zfsInfoMap[diskPerf.deviceId]
    poolInfo = zpoolInfoMap[dataSetInfo.pool]

    conf = GetConfig()
    quotaEnabled = getattr(conf, 'SunosDiskQuotaEnabled', False)

    if quotaEnabled:
        availBytes = parse_byte_value(dataSetInfo.avail)

        if dataSetInfo.refquota:
            referBytes = parse_byte_value(dataSetInfo.refer)
            refquotaBytes = parse_byte_value(dataSetInfo.refquota)
            diskPerf.usedSpace = referBytes
            diskPerf.freeSpace = min(availBytes, refquotaBytes - referBytes)
            diskPerf.totalSpace = refquotaBytes
        elif dataSetInfo.quota:
            referBytes = parse_byte_value(dataSetInfo.refer)
            quotaBytes = parse_byte_value(dataSetInfo.quota)
            diskPerf.usedSpace = referBytes
            diskPerf.freeSpace = min(availBytes, quotaBytes - referBytes)
            diskPerf.totalSpace = quotaBytes
        else:
            ancestorQuota = getAncestorQuota(dataSetInfo.name, zfsInfoMap)
            if ancestorQuota:
                quotaBytes = parse_byte_value(ancestorQuota)
                diskPerf.usedSpace = parse_byte_value(dataSetInfo.refer)
                diskPerf.freeSpace = availBytes
                diskPerf.totalSpace = quotaBytes
            else:
                diskPerf.totalSpace = parse_byte_value(poolInfo.totalSpace)
                diskPerf.usedSpace = parse_byte_value(dataSetInfo.refer)
                diskPerf.freeSpace = availBytes
    else:
        diskPerf.totalSpace = parse_byte_value(poolInfo.totalSpace)
        diskPerf.usedSpace = parse_byte_value(dataSetInfo.refer)
        diskPerf.freeSpace = parse_byte_value(dataSetInfo.avail)

    if diskPerf.totalSpace > 0:
        diskPerf.usedPercent = float(100.0 * float(diskPerf.usedSpace) / float(diskPerf.totalSpace))
        diskPerf.freePercent = float(100.0 * float(diskPerf.freeSpace) / float(diskPerf.totalSpace))
    else:
        diskPerf.usedPercent = float(0)
        diskPerf.freePercent = float(0)
    diskPerf.blksize = int(parse_byte_value(dataSetInfo.recordSize))
    diskPerf.readBps = poolInfo.readBps
    diskPerf.writeBps = poolInfo.writeBps
    diskPerf.readIops = poolInfo.readIops
    diskPerf.writeIops = poolInfo.writeIops
    diskPerf.ioPercent = poolInfo.ioPercent
    return diskPerf

def getDisk():
    f = open('/etc/mnttab','r')
    if not f:
        return None
    try:
        global zfsInfoCheck
        diskQuotas = []
        for l in f.readlines():
            words = l.split()
            if words[2] in filesystems:
                deviceId = words[0]
                p = _DiskPerf()
                p.mountPoint = words[1]
                p.deviceId = deviceId
                p.fileSystem = words[2]

                if words[2] == 'zfs' or words[2] == 'vzfs':
                    if zfsInfoCheck == False:
                        zfsInfoCheck = True
                        setZfsInfo()
                    p = getZFSDisk(p)
                    diskQuotas.append(p)
            
                else:
                    f_bsize, f_frsize, f_blocks, f_bfree, f_bavail, f_files, f_ffree, f_favail, f_flag, f_namemax = os.statvfs(p.mountPoint)
                    if f_blocks == 0:
                        continue
                    p.totalSpace = int(f_frsize * f_blocks)
                    p.usedSpace = int(f_frsize * (f_blocks - f_bfree))
                    p.freeSpace = p.totalSpace - p.usedSpace
                    p.usedPercent = float(100.0 * float(f_blocks - f_bfree) / float(f_blocks))
                    p.freePercent = float(100.0 * f_bfree / f_blocks)
                    p.blksize = int(f_frsize)
                    diskQuotas.append(p)
    finally:
        f.close()

    zfsInfoCheck = False
    return diskQuotas

class zfsIoStat:
    def __init__(self):
        self.name = None
        self.zfsType = None
        self.readOperation = 0
        self.writeOperation = 0
        self.readBandwidth = 0
        self.writeBandwidth = 0


class ZpoolPerfStat:
    def __init__(self):
        self.name = None
        self.poolName = None
        self.devName = None
        self.devType = None
        self.allocated = 0
        self.free = 0
        self.total = 0
        self.pctUsed = 0.0
        self.readBytes = 0
        self.writeBytes = 0
        self.readOps = 0
        self.writeOps = 0


zpoolInfo = {}
lock = threading.Lock()

_zpoolPerfSnapshot = {}
_zpoolPerfSnapshotLock = threading.Lock()

def safe_add_to_dict(key, value):
    global zpoolInfo
    with lock:
        zpoolInfo[key] = value

def safe_get_from_dict():
    global zpoolInfo
    ret = {}
    with lock:
        for k, v in zpoolInfo.items():
            ret[k] = v
            del zpoolInfo[k]
    return ret




#
#                              capacity     operations    bandwidth
#pool                        alloc   free   read  write   read  write
#-------------------------   -----  -----  -----  -----  -----  -----
#data-pool                   16.9G   539G      0      0      0      0
#  mirror-0                  16.9G   539G      0      0      0      0
#    c0t5000C500773ADFC3d0       -      -      0      0      0      0
#    c0t5000C50077331803d0       -      -      0      0      0      0
#-------------------------   -----  -----  -----  -----  -----  -----
#                              capacity     operations    bandwidth
#pool                        alloc   free   read  write   read  write
#-------------------------   -----  -----  -----  -----  -----  -----
#rpool                       90.4G  45.6G      0     51      0   144K
#  mirror-0                  90.4G  45.6G      0     51      0   144K
#    c1t5000039488127D22d0       -      -      0     16      0   145K
#    c1t5000039498033186d0       -      -      0     15      0   145K
#-------------------------   -----  -----  -----  -----  -----  -----

def parse_zpool_output(output, pool_re, diskMap):
    lines = output.strip().splitlines()
    current_pool = None
    current_type = "data"
    isPool = True
    parsed_data = {}


    for line in lines:
        if line.startswith("-"):
            continue

        line = line.strip()
        match = pool_re.match(line) 
        type_match = re.match(r'^\s*(\S+)\s*$', line)

        if match:
            groups = match.groupdict()
            name, alloc, free, read, write, read_bw, write_bw = (groups[key] for key in ["name", "alloc", "free", "read", "write", "read_bw", "write_bw"])
            if name ==  "pool" and alloc == "alloc" and free == "free": #header
                isPool = True
                current_type = "data"
                continue
            if name.startswith("mirror"):
                continue


            zfs_stat = zfsIoStat()
            if isPool:
                current_pool = name
                isPool = False
                zfs_stat.name = name 
            else:
                diskName = re.sub(r's\d+', '', name)
                if diskName in diskMap:
                    zfs_stat.name = "{}-{}({})".format(current_pool, diskName, diskMap[diskName])
                else:
                    zfs_stat.name = "{}-{}(Unknown)".format(current_pool, diskName)


            zfs_stat.zfsType = current_type
            zfs_stat.readOperation = parse_count_value(match.group("read"))
            zfs_stat.writeOperation = parse_count_value(match.group("write"))
            zfs_stat.readBandwidth = parse_byte_value(match.group("read_bw"))
            zfs_stat.writeBandwidth = parse_byte_value(match.group("write_bw"))

            safe_add_to_dict(zfs_stat.name, zfs_stat)

        if type_match:
            current_type = type_match.group(1)


def _getZpoolCapacity():
    command = "zpool list -H -o name,alloc,free,size"
    try:
        process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        text = process.stdout.read()
        cap_map = {}
        for line in text.strip('\n').split('\n'):
            if not line.strip():
                continue
            parts = line.split('\t')
            if len(parts) < 4:
                continue
            pool_name = parts[0]
            try:
                alloc = parse_byte_value(parts[1])
                free_bytes = parse_byte_value(parts[2])
                total = parse_byte_value(parts[3])
                cap_map[pool_name] = (alloc, free_bytes, total)
            except Exception:
                pass
        return cap_map
    except Exception as e:
        logging_util.error("zpool list capacity error: {} - {}".format(type(e).__name__, e))
        return {}


def _refreshZpoolPerfSnapshot(output, pool_re, diskMap, capacity_map):
    lines = output.strip().splitlines()
    current_pool = None
    current_type = "data"
    isPool = True
    new_snapshot = {}

    for line in lines:
        if line.startswith("-"):
            continue

        line = line.strip()
        match = pool_re.match(line)
        type_match = re.match(r'^\s*(\S+)\s*$', line)

        if match:
            groups = match.groupdict()
            name = groups["name"]

            if name == "pool" and groups["alloc"] == "alloc" and groups["free"] == "free":
                isPool = True
                current_type = "data"
                continue
            if name.startswith("mirror"):
                continue

            perf = ZpoolPerfStat()
            perf.devType = current_type

            if isPool:
                current_pool = name
                isPool = False
                perf.name = name
                perf.poolName = name
                perf.devName = name
                cap = capacity_map.get(name)
                if cap:
                    perf.allocated = cap[0]
                    perf.free = cap[1]
                    perf.total = cap[2]
                    denom = float(perf.allocated + perf.free)
                    if denom > 0:
                        perf.pctUsed = float(perf.allocated) / denom * 100.0
            else:
                diskName = re.sub(r's\d+', '', name)
                if diskName in diskMap:
                    perf.name = "{}-{}({})".format(current_pool, diskName, diskMap[diskName])
                else:
                    perf.name = "{}-{}(Unknown)".format(current_pool, diskName)
                perf.poolName = current_pool if current_pool else ""
                perf.devName = diskName

            try:
                perf.readOps = parse_count_value(groups["read"])
                perf.writeOps = parse_count_value(groups["write"])
                perf.readBytes = parse_byte_value(groups["read_bw"])
                perf.writeBytes = parse_byte_value(groups["write_bw"])
            except Exception:
                pass

            if perf.name:
                new_snapshot[perf.name] = perf

        if type_match:
            current_type = type_match.group(1)

    global _zpoolPerfSnapshot
    with _zpoolPerfSnapshotLock:
        _zpoolPerfSnapshot = new_snapshot


def GetZpoolPerfInfo():
    with _zpoolPerfSnapshotLock:
        return list(_zpoolPerfSnapshot.values())


def updateZFSPerf():
    pool_re = re.compile(r'^(?P<name>\S+)\s+(?P<alloc>\S+)\s+(?P<free>\S+)\s+(?P<read>\S+)\s+(?P<write>\S+)\s+(?P<read_bw>\S+)\s+(?P<write_bw>\S+)$')

    diskMap = getDiskMap()

    while True:
        process = subprocess.Popen(["zpool", "iostat", "-v", "5", "2"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, err = process.communicate()

        if err:
            return err

        intervals = output.split("\n\n")
        if len(intervals) >= 2:
            output = intervals[1]

        parse_zpool_output(output, pool_re, diskMap)

        capacity_map = _getZpoolCapacity()
        _refreshZpoolPerfSnapshot(output, pool_re, diskMap, capacity_map)


def initUpdateZfs():
    global _zfsUpdateStarted
    if _zfsUpdateStarted:
        return True
    f = open('/etc/mnttab','r')
    if not f:
        return None
    try:
        for l in f.readlines():
            words = l.split()
            if words[2] == 'zfs' or words[2] == 'vzfs':
                _zfsUpdateStarted = True
                thread_util.async(updateZFSPerf)
                break
    finally:
        f.close()

    return True

def GetZfsInfo():
    return safe_get_from_dict()



def test():
    for a in getDisk():
        print a
    time.sleep(10)
    for a in getDisk():
        print a


if __name__ == '__main__':
    test()




from whatap.pack.pack_enum import PackEnum
from whatap.pack.pack import Pack
from whatap.io.data_outputx import DataOutputX

from whatap.io.data_inputx import DataInputX

class SMNetPerfPack(Pack):
    def __init__(self):
        Pack.__init__(self)
        self.os = None
        self.netPerf = []

    def getPackType(self):
        return PackEnum.PACK_SM_NET_PERF

    def write(self, dout):
        super(SMNetPerfPack, self).write(dout)
        dout.writeShort(self.os)
        dout.writeDecimal(len(self.netPerf))
        for n in self.netPerf:
            n.write(dout)

    def read(self, din):
        din = DataInputX(din.readBlob())
        super(SMNetPerfPack, self).read(din)
        self.os = din.readShort()
        nicCount = din.readDecimal()
        for i in range(nicCount):
            n = NetPerf()
            n.read(din)
            self.netPerf.append(d)

class NetPerf(Pack):
    def __init__(self):
        self.desc = None
        self.ip = None
        self.hwAddr = None
        self.trafficIn = None
        self.trafficOut = None
        self.packetIn = None
        self.packetOut = None
        self.errorIn = None
        self.errorOut =  None
        self.droppedOut = None
        self.droppedIn = None

    def write(self, doutx ):
        dout= DataOutputX()
        dout.writeInt(self.desc)
        dout.writeBlob(self.ip)
        dout.writeText(self.hwAddr)
        dout.writeDouble(self.trafficIn)
        dout.writeDouble(self.trafficOut)
        dout.writeDouble(self.packetIn)
        dout.writeDouble(self.packetOut)
        dout.writeDouble(self.errorIn)
        dout.writeDouble(self.errorOut)
        dout.writeDouble(self.droppedOut)
        dout.writeDouble(self.droppedIn)

        dout.writeInt(1)

        doutx.writeBlob(dout.toByteArray())

    def read(self, dinx):
        din= DataInputX(dinx.readBlob())
        self.desc= din.ReadInt()
        self.ip= din.ReadInt()
        self.hwAddr = din.ReadInt()
        self.trafficIn = din.ReadDecimal()
        self.trafficOut = din.ReadDecimal()
        self.packetIn = din.ReadDecimal()
        self.packetOut = din.ReadFloat()
        self.errorIn = din.ReadFloat()
        self.errorOut = din.ReadInt()
        self.droppedOut = din.ReadDouble()
        self.droppedIn = din.ReadDouble()
        self.count = dinReadInt()




import os
import base64
import json
import time
import threading
import subprocess

try:
    from Queue import Queue, Empty
except ImportError:
    from queue import Queue, Empty

import whatap.agent.conf.configure as config
import whatap.util.logging_util as logging_util
from whatap.util.date_util import DateUtil as dateutil


# Script Manager Error Codes
ERR_SM_REQUEST_GENERAL       = "ER701000"
ERR_SM_UNKNOWN_REQ_METHOD    = "ER701001"
ERR_SM_ACTIVATE_JSON         = "ER701002"
ERR_SM_ACTIVATE_SCRIPT_FILE  = "ER701003"
ERR_SM_ACTIVATE_CRON_CONFIG  = "ER701004"
ERR_SM_DEACTIVATE_NO_JOBLIST = "ER701005"
ERR_SM_DELETE_NO_JOBLIST     = "ER701006"

ERR_SM_EXEC_GENERAL         = "ER702000"
ERR_SM_UNKNOWN_EXEC_METHOD  = "ER702001"
ERR_SM_EXEC_ENQUEUE_FAILED  = "ER702002"
ERR_SM_EXEC_NOT_INITIALIZED = "ER702003"
ERR_SM_EXEC_NO_UUID         = "ER702004"
ERR_SM_SCRIPT_DECODE_BASE64 = "ER702005"
ERR_SM_SCRIPT_TIMEOUT       = "ER702006"
ERR_SM_SCRIPT_EXECUTE       = "ER702007"

ERR_SM_STATUS_GENERAL         = "ER703000"
ERR_SM_STATUS_NOT_INITIALIZED = "ER703001"

ERR_SM_FILE_READ_ERROR = "ER704000"

ERR_SM_FILE_SAVE_GENERAL    = "ER705000"
ERR_SM_FILE_SAVE_ERROR      = "ER705001"
ERR_SM_FILE_SAVE_DONE_ERROR = "ER705002"
ERR_SM_FILE_SAVE_REQUEST    = "ER705004"

ERR_SM_FILE_LIST_GENERAL = "ER706000"
ERR_SM_FILE_LIST_ERROR   = "ER706001"

ERR_SM_FILE_DELETE_GENERAL = "ER707000"
ERR_SM_FILE_DELETE_ERROR   = "ER707001"


# Status constants
STATUS_WAITING  = "waiting"
STATUS_RUNNING  = "running"
STATUS_SUCCESS  = "success"
STATUS_ERROR    = "failed"
STATUS_NOTFOUND = "notfound"

# Method constants
METHOD_EXECUTE        = "execute"
METHOD_EXECUTE_RESULT = "executeResult"
METHOD_ACTIVATE       = "activate"
METHOD_DEACTIVATE     = "deactivate"
METHOD_DELETE         = "delete"

# Column constants
COLUMN_EXECUTE_UUID = "executeUuid"
COLUMN_CONTENT      = "content"
COLUMN_METHOD       = "method"
COLUMN_JOB_LIST     = "jobList"


class ScriptManagerError(Exception):
    def __init__(self, code, message, err=None):
        self.code = code
        self.message = message
        self.err = err
        super(ScriptManagerError, self).__init__(self._format_message())

    def _format_message(self):
        if self.err:
            return "[{}] {}: {}".format(self.code, self.message, self.err)
        return "[{}] {}".format(self.code, self.message)

    def error_code(self):
        return self.code


class ScriptResult(object):
    def __init__(self, stdout="", stderr="", exit_code=0, error=None, start_time=0, end_time=0):
        self.stdout = stdout
        self.stderr = stderr
        self.exit_code = exit_code
        self.error = error
        self.start_time = start_time
        self.end_time = end_time
        self.timestamp = time.time()


class ScriptResultStorage(object):
    def __init__(self, ttl=600):
        self.results = {}
        self.mutex = threading.Lock()
        self.ttl = ttl
        self._start_cleanup()

    def _start_cleanup(self):
        def cleanup_loop():
            while True:
                time.sleep(60)
                self._cleanup_expired()

        t = threading.Thread(target=cleanup_loop)
        t.daemon = True
        t.start()

    def _cleanup_expired(self):
        with self.mutex:
            now = time.time()
            expired = [uuid for uuid, result in self.results.items()
                       if now - result.timestamp > self.ttl]
            for uuid in expired:
                del self.results[uuid]

    def store(self, execute_uuid, stdout, stderr, exit_code, error, start_time, end_time):
        with self.mutex:
            self.results[execute_uuid] = ScriptResult(
                stdout=stdout,
                stderr=stderr,
                exit_code=exit_code,
                error=error,
                start_time=start_time,
                end_time=end_time
            )

    def get(self, execute_uuid):
        with self.mutex:
            return self.results.get(execute_uuid), execute_uuid in self.results

    def get_and_delete(self, execute_uuid):
        with self.mutex:
            result = self.results.get(execute_uuid)
            exists = execute_uuid in self.results
            if exists:
                del self.results[execute_uuid]
            return result, exists


class ScriptRunner(object):
    def __init__(self, capacity=10, timeout=5, result_ttl=600):
        self.queue = Queue(maxsize=capacity)
        self.script_timeout = timeout
        self.storage = ScriptResultStorage(ttl=result_ttl)
        self.queued_jobs = {}
        self.queue_mutex = threading.Lock()
        self.running_uuid = ""
        self.running_mutex = threading.Lock()
        self.running_start_time = 0
        self.max_queue_size = capacity
        self._running = False

    def start(self):
        self._running = True
        logging_util.info("[ScriptRunner] Started")

        def run_loop():
            while self._running:
                try:
                    try:
                        command_map = self.queue.get(timeout=0.1)
                    except Empty:
                        continue

                    execute_uuid = command_map.get(COLUMN_EXECUTE_UUID, "")
                    if not execute_uuid:
                        logging_util.error("[ScriptRunner] Missing executeUuid in command")
                        continue

                    with self.queue_mutex:
                        if execute_uuid in self.queued_jobs:
                            del self.queued_jobs[execute_uuid]

                    start_time = dateutil.now()
                    with self.running_mutex:
                        self.running_uuid = execute_uuid
                        self.running_start_time = start_time

                    command_base64 = command_map.get(COLUMN_CONTENT, "")
                    end_time = dateutil.now()

                    try:
                        command = base64.b64decode(command_base64)
                        if isinstance(command, bytes):
                            command = command.decode('utf-8', 'replace')
                    except Exception as e:
                        logging_util.error("[ScriptRunner] Base64 decode error: {}".format(e))
                        decode_err = ScriptManagerError(ERR_SM_SCRIPT_DECODE_BASE64,
                                                        "failed to decode base64 script content", e)
                        self.storage.store(execute_uuid, "", "", -1, decode_err, start_time, end_time)
                        with self.running_mutex:
                            self.running_uuid = ""
                        continue

                    stdout, stderr, exit_code, error = self._run_command(command, self.script_timeout)
                    end_time = dateutil.now()

                    max_output_length = 8000
                    if len(stdout) > max_output_length:
                        stdout = stdout[:max_output_length] + "..."
                    if len(stderr) > max_output_length:
                        stderr = stderr[:max_output_length] + "..."

                    if error:
                        logging_util.error("[ScriptRunner] Execute error: {}".format(error))
                        execute_err = ScriptManagerError(ERR_SM_SCRIPT_EXECUTE,
                                                         "failed to execute script", error)
                        self.storage.store(execute_uuid, stdout, stderr, exit_code, execute_err, start_time, end_time)
                    else:
                        logging_util.debug("[ScriptRunner] Execute success")
                        self.storage.store(execute_uuid, stdout, stderr, exit_code, None, start_time, end_time)

                    with self.running_mutex:
                        self.running_uuid = ""

                except Exception as e:
                    logging_util.error("[ScriptRunner] Loop error: {}".format(e))

        t = threading.Thread(target=run_loop)
        t.daemon = True
        t.start()

    def _run_command(self, command, timeout):
        stdout = ""
        stderr = ""
        exit_code = 0
        error = None
        output_data = [None, None]
        proc_error = [None]

        try:
            proc = subprocess.Popen(
                command,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                shell=True
            )

            def communicate_thread():
                try:
                    output_data[0], output_data[1] = proc.communicate()
                except Exception as e:
                    proc_error[0] = e

            comm_thread = threading.Thread(target=communicate_thread)
            comm_thread.start()
            comm_thread.join(timeout)

            if comm_thread.is_alive():
                proc.kill()
                proc.wait()
                error = Exception("Timeout after {} seconds".format(timeout))
                exit_code = -1
            elif proc_error[0]:
                error = proc_error[0]
                exit_code = -1
            else:
                exit_code = proc.returncode
                out = output_data[0]
                err = output_data[1]
                if out:
                    if isinstance(out, bytes):
                        stdout = out.decode('utf-8', 'replace')
                    else:
                        stdout = out
                if err:
                    if isinstance(err, bytes):
                        stderr = err.decode('utf-8', 'replace')
                    else:
                        stderr = err

        except Exception as e:
            error = e
            exit_code = -1

        return stdout, stderr, exit_code, error

    def enqueue(self, command_map):
        execute_uuid = command_map.get(COLUMN_EXECUTE_UUID, "")
        if not execute_uuid:
            raise ScriptManagerError(ERR_SM_EXEC_NO_UUID, "missing executeUuid in command", None)

        try:
            self.queue.put_nowait(command_map)
            with self.queue_mutex:
                self.queued_jobs[execute_uuid] = True
        except:
            raise ScriptManagerError(ERR_SM_EXEC_ENQUEUE_FAILED, "script runner queue full", None)

    def is_queued(self, execute_uuid):
        with self.queue_mutex:
            return execute_uuid in self.queued_jobs

    def is_running(self, execute_uuid):
        with self.running_mutex:
            return self.running_uuid == execute_uuid

    def get_queue_status(self):
        with self.queue_mutex:
            return {
                'current': len(self.queued_jobs),
                'total': self.max_queue_size
            }

    def get_running_start_time(self, execute_uuid):
        with self.running_mutex:
            if self.running_uuid == execute_uuid:
                return self.running_start_time
            return 0


# Global script runner instance
_script_runner = None


def init_script_manager():
    global _script_runner
    conf = config.GetConfig()
    queue_size = getattr(conf, 'ScriptManagerExecQueueSize', 10)
    timeout = getattr(conf, 'CronjobTimeout', 5)
    result_ttl = 600  # 10 minutes

    _script_runner = ScriptRunner(capacity=queue_size, timeout=timeout, result_ttl=result_ttl)
    _script_runner.start()
    logging_util.info("[ScriptManager] Initialized")


def execute_script(command_map):
    global _script_runner
    if _script_runner is None:
        raise ScriptManagerError(ERR_SM_EXEC_NOT_INITIALIZED, "script runner not initialized", None)

    method = command_map.get(COLUMN_METHOD, "")

    if method == METHOD_EXECUTE:
        _script_runner.enqueue(command_map)
        return None

    elif method == METHOD_EXECUTE_RESULT:
        execute_uuid = command_map.get(COLUMN_EXECUTE_UUID, "")
        if not execute_uuid:
            raise ScriptManagerError(ERR_SM_EXEC_NO_UUID, "missing executeUuid for executeResult", None)
        return get_script_status(execute_uuid)

    else:
        raise ScriptManagerError(ERR_SM_UNKNOWN_EXEC_METHOD,
                                 "invalid method for execution: {}".format(method), None)


def get_script_status(execute_uuid):
    global _script_runner
    if _script_runner is None:
        raise ScriptManagerError(ERR_SM_EXEC_NOT_INITIALIZED, "script runner not initialized", None)

    queue_status = _script_runner.get_queue_status()

    # 1. Check if running
    if _script_runner.is_running(execute_uuid):
        start_time = _script_runner.get_running_start_time(execute_uuid)
        return {
            'uuid': execute_uuid,
            'status': STATUS_RUNNING,
            'queue_status': queue_status,
            'start_time': start_time
        }

    # 2. Check if queued
    if _script_runner.is_queued(execute_uuid):
        return {
            'uuid': execute_uuid,
            'status': STATUS_WAITING,
            'queue_status': queue_status
        }

    # 3. Check completed results
    result, exists = _script_runner.storage.get_and_delete(execute_uuid)
    if exists:
        status = STATUS_SUCCESS if result.exit_code == 0 else STATUS_ERROR
        response = {
            'uuid': execute_uuid,
            'status': status,
            'queue_status': queue_status,
            'start_time': result.start_time,
            'end_time': result.end_time,
            'stdout': result.stdout,
            'stderr': result.stderr,
            'exit_code': result.exit_code
        }
        if result.error:
            response['error'] = result.error
        return response

    # 4. Not found
    return {
        'uuid': execute_uuid,
        'status': STATUS_NOTFOUND,
        'queue_status': queue_status
    }


def get_running_script_status():
    global _script_runner
    if _script_runner is None:
        raise ScriptManagerError(ERR_SM_EXEC_NOT_INITIALIZED, "script runner not initialized", None)

    queue_status = _script_runner.get_queue_status()

    with _script_runner.running_mutex:
        running_uuid = _script_runner.running_uuid
        running_start_time = _script_runner.running_start_time

    if running_uuid:
        return {
            'uuid': running_uuid,
            'status': STATUS_RUNNING,
            'start_time': running_start_time,
            'queue_status': queue_status
        }

    return {
        'uuid': '',
        'status': STATUS_NOTFOUND,
        'queue_status': queue_status
    }


def read_file(file_path, pos, length, direction="forward"):
    try:
        if not os.path.exists(file_path):
            raise Exception("File not found: {}".format(file_path))

        file_size = os.path.getsize(file_path)

        with open(file_path, 'r') as f:
            if direction == "backward":
                start_pos = max(0, pos - length)
                f.seek(start_pos)
                text = f.read(min(length, pos))
                end_pos = pos
            else:
                f.seek(pos)
                text = f.read(length)
                start_pos = pos
                end_pos = pos + len(text)

        return text, start_pos, end_pos, file_size, None

    except Exception as e:
        return "", 0, 0, 0, e


def list_files():
    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')

    try:
        if not os.path.exists(cron_dir):
            return [], None

        files = []
        for f in os.listdir(cron_dir):
            full_path = os.path.join(cron_dir, f)
            if os.path.isfile(full_path):
                files.append(f)
        return files, None

    except Exception as e:
        return [], e


def delete_file(file_name):
    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')
    file_path = os.path.join(cron_dir, os.path.basename(file_name))

    try:
        if os.path.exists(file_path):
            os.remove(file_path)
        return None
    except Exception as e:
        return e


# File save state for chunked uploads
_file_save_state = {}
_file_save_mutex = threading.Lock()


def save_file_chunk(file_name, pos, data):
    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')

    if not os.path.exists(cron_dir):
        try:
            os.makedirs(cron_dir)
        except Exception as e:
            raise ScriptManagerError(ERR_SM_FILE_SAVE_ERROR,
                                     "failed to create directory: {}".format(cron_dir), e)

    file_path = os.path.join(cron_dir, os.path.basename(file_name))
    tmp_path = file_path + ".tmp"

    try:
        mode = 'ab' if pos > 0 else 'wb'
        with open(tmp_path, mode) as f:
            if pos > 0:
                f.seek(pos)
            f.write(data)

        with _file_save_mutex:
            _file_save_state[file_name] = tmp_path

        return None

    except Exception as e:
        raise ScriptManagerError(ERR_SM_FILE_SAVE_ERROR,
                                 "failed to save file chunk", e)


def save_file_done(file_name, size, crc):
    conf = config.GetConfig()
    cron_dir = getattr(conf, 'CronjobPath', '/usr/whatap/infra/cronjob')
    file_path = os.path.join(cron_dir, os.path.basename(file_name))
    tmp_path = file_path + ".tmp"

    try:
        if os.path.exists(tmp_path):
            # Verify size
            actual_size = os.path.getsize(tmp_path)
            if actual_size != size:
                os.remove(tmp_path)
                raise ScriptManagerError(ERR_SM_FILE_SAVE_DONE_ERROR,
                                         "size mismatch: expected {}, got {}".format(size, actual_size), None)

            # Make executable on unix
            os.chmod(tmp_path, 0o755)

            # Atomic rename
            if os.path.exists(file_path):
                os.remove(file_path)
            os.rename(tmp_path, file_path)

            with _file_save_mutex:
                if file_name in _file_save_state:
                    del _file_save_state[file_name]

            return None
        else:
            raise ScriptManagerError(ERR_SM_FILE_SAVE_DONE_ERROR,
                                     "temp file not found: {}".format(tmp_path), None)

    except ScriptManagerError:
        raise
    except Exception as e:
        raise ScriptManagerError(ERR_SM_FILE_SAVE_DONE_ERROR,
                                 "failed to finalize file save", e)



import unittest
import Tkinter as tkinter
import tkFont as font
from test.test_support import requires, run_unittest
from test_ttk.support import AbstractTkTest

requires('gui')

class FontTest(AbstractTkTest, unittest.TestCase):

    def test_font_eq(self):
        fontname = "TkDefaultFont"
        try:
            f = font.Font(root=self.root, name=fontname, exists=True)
        except tkinter._tkinter.TclError:
            f = font.Font(root=self.root, name=fontname, exists=False)
        font1 = font.Font(root=self.root, name=fontname, exists=True)
        font2 = font.Font(root=self.root, name=fontname, exists=True)
        self.assertIsNot(font1, font2)
        self.assertEqual(font1, font2)
        self.assertNotEqual(font1, font1.copy())
        self.assertNotEqual(font1, 0)
        self.assertNotIn(font1, [0])

tests_gui = (FontTest, )

if __name__ == "__main__":
    run_unittest(*tests_gui)



from lib2to3.fixer_base import BaseFix
from lib2to3.fixer_util import Name

class FixParrot(BaseFix):
    """
    Change functions named 'parrot' to 'cheese'.
    """

    PATTERN = """funcdef < 'def' name='parrot' any* >"""

    def transform(self, node, results):
        name = results["name"]
        name.replace(Name("cheese", name.prefix))



""" Python 'hex_codec' Codec - 2-digit hex content transfer encoding

    Unlike most of the other codecs which target Unicode, this codec
    will return Python string objects for both encode and decode.

    Written by Marc-Andre Lemburg (mal@lemburg.com).

"""
import codecs, binascii

### Codec APIs

def hex_encode(input,errors='strict'):

    """ Encodes the object input and returns a tuple (output
        object, length consumed).

        errors defines the error handling to apply. It defaults to
        'strict' handling which is the only currently supported
        error handling for this codec.

    """
    assert errors == 'strict'
    output = binascii.b2a_hex(input)
    return (output, len(input))

def hex_decode(input,errors='strict'):

    """ Decodes the object input and returns a tuple (output
        object, length consumed).

        input must be an object which provides the bf_getreadbuf
        buffer slot. Python strings, buffer objects and memory
        mapped files are examples of objects providing this slot.

        errors defines the error handling to apply. It defaults to
        'strict' handling which is the only currently supported
        error handling for this codec.

    """
    assert errors == 'strict'
    output = binascii.a2b_hex(input)
    return (output, len(input))

class Codec(codecs.Codec):

    def encode(self, input,errors='strict'):
        return hex_encode(input,errors)
    def decode(self, input,errors='strict'):
        return hex_decode(input,errors)

class IncrementalEncoder(codecs.IncrementalEncoder):
    def encode(self, input, final=False):
        assert self.errors == 'strict'
        return binascii.b2a_hex(input)

class IncrementalDecoder(codecs.IncrementalDecoder):
    def decode(self, input, final=False):
        assert self.errors == 'strict'
        return binascii.a2b_hex(input)

class StreamWriter(Codec,codecs.StreamWriter):
    pass

class StreamReader(Codec,codecs.StreamReader):
    pass

### encodings module API

def getregentry():
    return codecs.CodecInfo(
        name='hex',
        encode=hex_encode,
        decode=hex_decode,
        incrementalencoder=IncrementalEncoder,
        incrementaldecoder=IncrementalDecoder,
        streamwriter=StreamWriter,
        streamreader=StreamReader,
    )



from ctypes import *
import unittest
import struct

def valid_ranges(*types):
    # given a sequence of numeric types, collect their _type_
    # attribute, which is a single format character compatible with
    # the struct module, use the struct module to calculate the
    # minimum and maximum value allowed for this format.
    # Returns a list of (min, max) values.
    result = []
    for t in types:
        fmt = t._type_
        size = struct.calcsize(fmt)
        a = struct.unpack(fmt, ("\x00"*32)[:size])[0]
        b = struct.unpack(fmt, ("\xFF"*32)[:size])[0]
        c = struct.unpack(fmt, ("\x7F"+"\x00"*32)[:size])[0]
        d = struct.unpack(fmt, ("\x80"+"\xFF"*32)[:size])[0]
        result.append((min(a, b, c, d), max(a, b, c, d)))
    return result

ArgType = type(byref(c_int(0)))

unsigned_types = [c_ubyte, c_ushort, c_uint, c_ulong]
signed_types = [c_byte, c_short, c_int, c_long, c_longlong]

bool_types = []

float_types = [c_double, c_float]

try:
    c_ulonglong
    c_longlong
except NameError:
    pass
else:
    unsigned_types.append(c_ulonglong)
    signed_types.append(c_longlong)

try:
    c_bool
except NameError:
    pass
else:
    bool_types.append(c_bool)

unsigned_ranges = valid_ranges(*unsigned_types)
signed_ranges = valid_ranges(*signed_types)
bool_values = [True, False, 0, 1, -1, 5000, 'test', [], [1]]

################################################################

class NumberTestCase(unittest.TestCase):

    def test_default_init(self):
        # default values are set to zero
        for t in signed_types + unsigned_types + float_types:
            self.assertEqual(t().value, 0)

    def test_unsigned_values(self):
        # the value given to the constructor is available
        # as the 'value' attribute
        for t, (l, h) in zip(unsigned_types, unsigned_ranges):
            self.assertEqual(t(l).value, l)
            self.assertEqual(t(h).value, h)

    def test_signed_values(self):
        # see above
        for t, (l, h) in zip(signed_types, signed_ranges):
            self.assertEqual(t(l).value, l)
            self.assertEqual(t(h).value, h)

    def test_bool_values(self):
        from operator import truth
        for t, v in zip(bool_types, bool_values):
            self.assertEqual(t(v).value, truth(v))

    def test_typeerror(self):
        # Only numbers are allowed in the contructor,
        # otherwise TypeError is raised
        for t in signed_types + unsigned_types + float_types:
            self.assertRaises(TypeError, t, "")
            self.assertRaises(TypeError, t, None)

    @unittest.skip('test disabled')
    def test_valid_ranges(self):
        # invalid values of the correct type
        # raise ValueError (not OverflowError)
        for t, (l, h) in zip(unsigned_types, unsigned_ranges):
            self.assertRaises(ValueError, t, l-1)
            self.assertRaises(ValueError, t, h+1)

    def test_from_param(self):
        # the from_param class method attribute always
        # returns PyCArgObject instances
        for t in signed_types + unsigned_types + float_types:
            self.assertEqual(ArgType, type(t.from_param(0)))

    def test_byref(self):
        # calling byref returns also a PyCArgObject instance
        for t in signed_types + unsigned_types + float_types + bool_types:
            parm = byref(t())
            self.assertEqual(ArgType, type(parm))


    def test_floats(self):
        # c_float and c_double can be created from
        # Python int, long and float
        class FloatLike(object):
            def __float__(self):
                return 2.0
        f = FloatLike()
        for t in float_types:
            self.assertEqual(t(2.0).value, 2.0)
            self.assertEqual(t(2).value, 2.0)
            self.assertEqual(t(2L).value, 2.0)
            self.assertEqual(t(f).value, 2.0)

    def test_integers(self):
        class FloatLike(object):
            def __float__(self):
                return 2.0
        f = FloatLike()
        class IntLike(object):
            def __int__(self):
                return 2
        i = IntLike()
        # integers cannot be constructed from floats,
        # but from integer-like objects
        for t in signed_types + unsigned_types:
            self.assertRaises(TypeError, t, 3.14)
            self.assertRaises(TypeError, t, f)
            self.assertEqual(t(i).value, 2)

    def test_sizes(self):
        for t in signed_types + unsigned_types + float_types + bool_types:
            try:
                size = struct.calcsize(t._type_)
            except struct.error:
                continue
            # sizeof of the type...
            self.assertEqual(sizeof(t), size)
            # and sizeof of an instance
            self.assertEqual(sizeof(t()), size)

    def test_alignments(self):
        for t in signed_types + unsigned_types + float_types:
            code = t._type_ # the typecode
            align = struct.calcsize("c%c" % code) - struct.calcsize(code)

            # alignment of the type...
            self.assertEqual((code, alignment(t)),
                                 (code, align))
            # and alignment of an instance
            self.assertEqual((code, alignment(t())),
                                 (code, align))

    def test_int_from_address(self):
        from array import array
        for t in signed_types + unsigned_types:
            # the array module doesn't support all format codes
            # (no 'q' or 'Q')
            try:
                array(t._type_)
            except ValueError:
                continue
            a = array(t._type_, [100])

            # v now is an integer at an 'external' memory location
            v = t.from_address(a.buffer_info()[0])
            self.assertEqual(v.value, a[0])
            self.assertEqual(type(v), t)

            # changing the value at the memory location changes v's value also
            a[0] = 42
            self.assertEqual(v.value, a[0])


    def test_float_from_address(self):
        from array import array
        for t in float_types:
            a = array(t._type_, [3.14])
            v = t.from_address(a.buffer_info()[0])
            self.assertEqual(v.value, a[0])
            self.assertIs(type(v), t)
            a[0] = 2.3456e17
            self.assertEqual(v.value, a[0])
            self.assertIs(type(v), t)

    def test_char_from_address(self):
        from ctypes import c_char
        from array import array

        a = array('c', 'x')
        v = c_char.from_address(a.buffer_info()[0])
        self.assertEqual(v.value, a[0])
        self.assertIs(type(v), c_char)

        a[0] = '?'
        self.assertEqual(v.value, a[0])

    # array does not support c_bool / 't'
    @unittest.skip('test disabled')
    def test_bool_from_address(self):
        from ctypes import c_bool
        from array import array
        a = array(c_bool._type_, [True])
        v = t.from_address(a.buffer_info()[0])
        self.assertEqual(v.value, a[0])
        self.assertEqual(type(v) is t)
        a[0] = False
        self.assertEqual(v.value, a[0])
        self.assertEqual(type(v) is t)

    def test_init(self):
        # c_int() can be initialized from Python's int, and c_int.
        # Not from c_long or so, which seems strange, abc should
        # probably be changed:
        self.assertRaises(TypeError, c_int, c_long(42))

    def test_float_overflow(self):
        import sys
        big_int = int(sys.float_info.max) * 2
        for t in float_types + [c_longdouble]:
            self.assertRaises(OverflowError, t, big_int)
            if (hasattr(t, "__ctype_be__")):
                self.assertRaises(OverflowError, t.__ctype_be__, big_int)
            if (hasattr(t, "__ctype_le__")):
                self.assertRaises(OverflowError, t.__ctype_le__, big_int)

    @unittest.skip('test disabled')
    def test_perf(self):
        check_perf()

from ctypes import _SimpleCData
class c_int_S(_SimpleCData):
    _type_ = "i"
    __slots__ = []

def run_test(rep, msg, func, arg=None):
##    items = [None] * rep
    items = range(rep)
    from time import clock
    if arg is not None:
        start = clock()
        for i in items:
            func(arg); func(arg); func(arg); func(arg); func(arg)
        stop = clock()
    else:
        start = clock()
        for i in items:
            func(); func(); func(); func(); func()
        stop = clock()
    print "%15s: %.2f us" % (msg, ((stop-start)*1e6/5/rep))

def check_perf():
    # Construct 5 objects
    from ctypes import c_int

    REP = 200000

    run_test(REP, "int()", int)
    run_test(REP, "int(999)", int)
    run_test(REP, "c_int()", c_int)
    run_test(REP, "c_int(999)", c_int)
    run_test(REP, "c_int_S()", c_int_S)
    run_test(REP, "c_int_S(999)", c_int_S)

# Python 2.3 -OO, win2k, P4 700 MHz:
#
#          int(): 0.87 us
#       int(999): 0.87 us
#        c_int(): 3.35 us
#     c_int(999): 3.34 us
#      c_int_S(): 3.23 us
#   c_int_S(999): 3.24 us

# Python 2.2 -OO, win2k, P4 700 MHz:
#
#          int(): 0.89 us
#       int(999): 0.89 us
#        c_int(): 9.99 us
#     c_int(999): 10.02 us
#      c_int_S(): 9.87 us
#   c_int_S(999): 9.85 us

if __name__ == '__main__':
##    check_perf()
    unittest.main()



import os, time
import whatap.util.process_util as process_util
from whatap.util.date_util import DateUtil as date_util
import whatap.util.thread_util as thread_util
import whatap.util.logging_util as logging_util
import whatap.agent.conf.configure as config
import whatap.agent.secure.security_master as secure
import whatap.pack.tagcount_pack as tagcount_pack
import whatap.agent.data.data_pack as data
from StringIO import StringIO


def check():
    pass

def testRun():
    import whatap.agent.conf.configure as config
    m= config.GetConfig().AgentlessItems
    if not m:
        return

    prefix = config.entrypoint_path
    for k, item in m.items():
        # logging_util.debug("testRun ", prefix, config.ScriptPath, item.script)
        check(item, os.path.join(prefix, config.ScriptPath, item.script), console_output=True)


class NameValuePair (object):
    def __init__(self, name = None, value=None):
        self.name = name
        self.value = value

class ClockValuePair (object):
    def __init__(self, clock = None, value= None):
        self.clock = clock
        self.value = value

class SimpleFloatMap(dict):
    def __init__(self):
        super(SimpleFloatMap, self).__init__()

class SimpleStringMap(dict):
    def __init__(self):
        super(SimpleStringMap, self).__init__()

class SimpleClockValueMap(dict):
    def __init__(self):
        super(SimpleClockValueMap, self).__init__()

#type SimpleFloatMap map[string]*map[string]float64
#type SimpleStringMap map[string][]NameValuePair
#type SimpleClockValueMap map[string]*ClockValuePair

def check(item , script , console_output=False) :
    logging_util.debug("check step -1 ", item.uid, item.gid , script)
    mapvalues = {}
    metavalues = {}
    if console_output:
        print('executing ', script ,' as uid:', item.uid, ' gid:', item.gid)
    exitcode, output = process_util.executeCmdEx(script, uid= item.uid, gid= item.gid)
    print('exitcode ', exitcode)
    if console_output:
        print(output)
    scanner = StringIO(output)
    for line in scanner.readlines():
        words = line.split()
        if console_output:
            print(words)
        if len(words) < 4 :
            continue

        datatype = words[0]
        instance = words[1]
        name = words[2]

        logging_util.debug("check step -2 ", words)
        # logging_util.debug("check step -2 ", id(metavalues), metavalues)
        if "M" == datatype or "m" == datatype :
            # logging_util.debug("check step -2.1 ", id(metavalues), metavalues)
            metaValue = " ".join(words[3:])
            # logging_util.debug("check step -2.2 ", id(metavalues), metavalues)
            if instance in metavalues:
                # logging_util.debug("check step -2.3 ", id(metavalues), metavalues)
                v = metavalues[instance]
                # logging_util.debug("check step -2.4 ", id(metavalues), metavalues)
                v.append(NameValuePair(name=name, value=metaValue))

            else:
                # logging_util.debug("check step -2.6 ", id(metavalues), metavalues)
                metavalues[instance] = [ NameValuePair(name= name, value= metaValue)]
                # logging_util.debug("check step -2.7 ", id(metavalues), metavalues)
        elif "H" == datatype or "h" == datatype:
            v = float(words[3])

            if instance not in  mapvalues:
                mapvalues[instance] = {}

            instValue = mapvalues[instance]
            instValue[name] = v
        # logging_util.debug("check step -3 ", id(metavalues), metavalues)

    return mapvalues, metavalues, 0


def start():
    thread_util.asyncLoop(_process, 30)


def _process():
    global concurrent
    m = {}
    m.update(config.GetConfig().AgentlessItems)
    logging_util.debug("checkmain._process step -1", m)
    if not m:
        return
    # logging_util.debug("checkmain._process step -2")
    secu = secure.GetSecurityMaster()
    now = date_util.now()

    def probe(pitem):
        logging_util.debug("_process probe ", item, config.entrypoint_path, config.ScriptPath, pitem.script)
        mapvalues, metavalues, _ = check(pitem, os.path.join(config.entrypoint_path, config.ScriptPath, pitem.script))

        logging_util.debug("_process step -5", metavalues)
        for inst, nameNvalues in mapvalues.items():
            p = tagcount_pack.TagCountPack()
            p.category = pitem.script.replace("/", "_").replace(".", "_")
            p.pcode = secu.pcode
            p.oid = secu.oid
            p.time = now
            for f, fvalue in nameNvalues.items():
                p.fields.putFloat(f, fvalue)
            p.tags.putString('inst', inst)
            if inst in metavalues:
                for nameNvalue in metavalues[inst]:
                    p.tags.putString(nameNvalue.name, nameNvalue.value)
                    # panicutil.Debug("_process step -5.0.2  ", nameNvalue.Name, nameNvalue.Value)
            # // panicutil.Debug("_process step -5.1 sending ", item.Script)
            logging_util.debug("checkmain._process step -3", p)
            data.SendHide(p)

    concurrent = 0
    prefix = config.entrypoint_path
    threads = []
    for item in m.values():
        # logging_util.debug("checkmain._process step -3")
        while concurrent >= config.GetConfig().AgentlessConcurrent:
            while threads:
                t = threads.pop()
                t.join()

        # logging_util.debug("checkmain._process step -4")
        concurrent +=1
        t = thread_util.async(probe, item)
        threads.append(t)
    while threads:
        t = threads.pop()
        t.join()



doctests = """
########### Tests mostly copied from test_listcomps.py ############

Test simple loop with conditional

    >>> sum({i*i for i in range(100) if i&1 == 1})
    166650

Test simple case

    >>> {2*y + x + 1 for x in (0,) for y in (1,)}
    set([3])

Test simple nesting

    >>> list(sorted({(i,j) for i in range(3) for j in range(4)}))
    [(0, 0), (0, 1), (0, 2), (0, 3), (1, 0), (1, 1), (1, 2), (1, 3), (2, 0), (2, 1), (2, 2), (2, 3)]

Test nesting with the inner expression dependent on the outer

    >>> list(sorted({(i,j) for i in range(4) for j in range(i)}))
    [(1, 0), (2, 0), (2, 1), (3, 0), (3, 1), (3, 2)]

Make sure the induction variable is not exposed

    >>> i = 20
    >>> sum({i*i for i in range(100)})
    328350

    >>> i
    20

Verify that syntax error's are raised for setcomps used as lvalues

    >>> {y for y in (1,2)} = 10          # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
       ...
    SyntaxError: ...

    >>> {y for y in (1,2)} += 10         # doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
       ...
    SyntaxError: ...


Make a nested set comprehension that acts like set(range())

    >>> def srange(n):
    ...     return {i for i in range(n)}
    >>> list(sorted(srange(10)))
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

Same again, only as a lambda expression instead of a function definition

    >>> lrange = lambda n:  {i for i in range(n)}
    >>> list(sorted(lrange(10)))
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

Generators can call other generators:

    >>> def grange(n):
    ...     for x in {i for i in range(n)}:
    ...         yield x
    >>> list(sorted(grange(5)))
    [0, 1, 2, 3, 4]


Make sure that None is a valid return value

    >>> {None for i in range(10)}
    set([None])

########### Tests for various scoping corner cases ############

Return lambdas that use the iteration variable as a default argument

    >>> items = {(lambda i=i: i) for i in range(5)}
    >>> {x() for x in items} == set(range(5))
    True

Same again, only this time as a closure variable

    >>> items = {(lambda: i) for i in range(5)}
    >>> {x() for x in items}
    set([4])

Another way to test that the iteration variable is local to the list comp

    >>> items = {(lambda: i) for i in range(5)}
    >>> i = 20
    >>> {x() for x in items}
    set([4])

And confirm that a closure can jump over the list comp scope

    >>> items = {(lambda: y) for i in range(5)}
    >>> y = 2
    >>> {x() for x in items}
    set([2])

We also repeat each of the above scoping tests inside a function

    >>> def test_func():
    ...     items = {(lambda i=i: i) for i in range(5)}
    ...     return {x() for x in items}
    >>> test_func() == set(range(5))
    True

    >>> def test_func():
    ...     items = {(lambda: i) for i in range(5)}
    ...     return {x() for x in items}
    >>> test_func()
    set([4])

    >>> def test_func():
    ...     items = {(lambda: i) for i in range(5)}
    ...     i = 20
    ...     return {x() for x in items}
    >>> test_func()
    set([4])

    >>> def test_func():
    ...     items = {(lambda: y) for i in range(5)}
    ...     y = 2
    ...     return {x() for x in items}
    >>> test_func()
    set([2])

"""


__test__ = {'doctests' : doctests}

def test_main(verbose=None):
    import sys
    from test import test_support
    from test import test_setcomps
    test_support.run_doctest(test_setcomps, verbose)

    # verify reference counting
    if verbose and hasattr(sys, "gettotalrefcount"):
        import gc
        counts = [None] * 5
        for i in range(len(counts)):
            test_support.run_doctest(test_setcomps, verbose)
            gc.collect()
            counts[i] = sys.gettotalrefcount()
        print(counts)

if __name__ == "__main__":
    test_main(verbose=True)




import logging
import logging.handlers
from logging.handlers import RotatingFileHandler
import inspect

infraLogger = None

def initLogger(logPath, logLevel = logging.INFO, logMaxBytes = 4 * 1024 * 1024, logBackupCount = 3):
    logger = logging.getLogger("InfraLogger")
    logger.setLevel(logLevel)

    formatter = logging.Formatter('[%(asctime)s] [%(levelname)s] [%(wfilename)s:%(wfuncName)s:%(wlineno)d] %(message)s ')

    handler = RotatingFileHandler(
            filename=logPath,
            maxBytes=logMaxBytes,
            backupCount=logBackupCount
            )

    handler.setFormatter(formatter)

    if logger.handlers:
        logger.handlers = []

    logger.addHandler(handler)

    global infraLogger
    infraLogger = logger



def debug(*args):
    frame = inspect.stack()[1]
    filename = frame[1]
    lineno = frame[2]
    funcname = frame[3]

    message = " ".join([str(x) for x in args])
    extra = {
            'wfilename': filename,
            'wlineno': lineno, 
            'wfuncName': funcname
    }

    infraLogger.debug(message, extra = extra)

def info(*args):
    frame = inspect.stack()[1]
    dir(frame)
    filename = frame[1]
    lineno = frame[2]
    funcname = frame[3]

    message = " ".join([str(x) for x in args])
    extra = {
            'wfilename': filename,
            'wlineno': lineno, 
            'wfuncName': funcname
    }

    infraLogger.info(message, extra = extra)

def error(*args):
    frame = inspect.stack()[1]
    filename = frame[1]
    lineno = frame[2]
    funcname = frame[3]

    message = " ".join([str(x) for x in args])
    extra = {
            'wfilename': filename,
            'wlineno': lineno, 
            'wfuncName': funcname
    }
    print extra
    print message

    infraLogger.error(message, extra = extra)


def example_function():
    for i in range(0, 1000000000000000000000000000000):
        info("This is an INFO message from example_function.")
        error("This is an ERROR message from example_function.")
        debug("This is a DEBUG message.")


initLogger("./test.log")
example_function()




# -*- coding: iso-8859-1 -*-
import unittest
import shlex

from test import test_support

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


# The original test data set was from shellwords, by Hartmut Goebel.

data = r"""x|x|
foo bar|foo|bar|
 foo bar|foo|bar|
 foo bar |foo|bar|
foo   bar    bla     fasel|foo|bar|bla|fasel|
x y  z              xxxx|x|y|z|xxxx|
\x bar|\|x|bar|
\ x bar|\|x|bar|
\ bar|\|bar|
foo \x bar|foo|\|x|bar|
foo \ x bar|foo|\|x|bar|
foo \ bar|foo|\|bar|
foo "bar" bla|foo|"bar"|bla|
"foo" "bar" "bla"|"foo"|"bar"|"bla"|
"foo" bar "bla"|"foo"|bar|"bla"|
"foo" bar bla|"foo"|bar|bla|
foo 'bar' bla|foo|'bar'|bla|
'foo' 'bar' 'bla'|'foo'|'bar'|'bla'|
'foo' bar 'bla'|'foo'|bar|'bla'|
'foo' bar bla|'foo'|bar|bla|
blurb foo"bar"bar"fasel" baz|blurb|foo"bar"bar"fasel"|baz|
blurb foo'bar'bar'fasel' baz|blurb|foo'bar'bar'fasel'|baz|
""|""|
''|''|
foo "" bar|foo|""|bar|
foo '' bar|foo|''|bar|
foo "" "" "" bar|foo|""|""|""|bar|
foo '' '' '' bar|foo|''|''|''|bar|
\""|\|""|
"\"|"\"|
"foo\ bar"|"foo\ bar"|
"foo\\ bar"|"foo\\ bar"|
"foo\\ bar\"|"foo\\ bar\"|
"foo\\" bar\""|"foo\\"|bar|\|""|
"foo\\ bar\" dfadf"|"foo\\ bar\"|dfadf"|
"foo\\\ bar\" dfadf"|"foo\\\ bar\"|dfadf"|
"foo\\\x bar\" dfadf"|"foo\\\x bar\"|dfadf"|
"foo\x bar\" dfadf"|"foo\x bar\"|dfadf"|
\''|\|''|
'foo\ bar'|'foo\ bar'|
'foo\\ bar'|'foo\\ bar'|
"foo\\\x bar\" df'a\ 'df'|"foo\\\x bar\"|df'a|\|'df'|
\"foo"|\|"foo"|
\"foo"\x|\|"foo"|\|x|
"foo\x"|"foo\x"|
"foo\ "|"foo\ "|
foo\ xx|foo|\|xx|
foo\ x\x|foo|\|x|\|x|
foo\ x\x\""|foo|\|x|\|x|\|""|
"foo\ x\x"|"foo\ x\x"|
"foo\ x\x\\"|"foo\ x\x\\"|
"foo\ x\x\\""foobar"|"foo\ x\x\\"|"foobar"|
"foo\ x\x\\"\''"foobar"|"foo\ x\x\\"|\|''|"foobar"|
"foo\ x\x\\"\'"fo'obar"|"foo\ x\x\\"|\|'"fo'|obar"|
"foo\ x\x\\"\'"fo'obar" 'don'\''t'|"foo\ x\x\\"|\|'"fo'|obar"|'don'|\|''|t'|
'foo\ bar'|'foo\ bar'|
'foo\\ bar'|'foo\\ bar'|
foo\ bar|foo|\|bar|
foo#bar\nbaz|foobaz|
:-) ;-)|:|-|)|;|-|)|
áéíóú|á|é|í|ó|ú|
"""

posix_data = r"""x|x|
foo bar|foo|bar|
 foo bar|foo|bar|
 foo bar |foo|bar|
foo   bar    bla     fasel|foo|bar|bla|fasel|
x y  z              xxxx|x|y|z|xxxx|
\x bar|x|bar|
\ x bar| x|bar|
\ bar| bar|
foo \x bar|foo|x|bar|
foo \ x bar|foo| x|bar|
foo \ bar|foo| bar|
foo "bar" bla|foo|bar|bla|
"foo" "bar" "bla"|foo|bar|bla|
"foo" bar "bla"|foo|bar|bla|
"foo" bar bla|foo|bar|bla|
foo 'bar' bla|foo|bar|bla|
'foo' 'bar' 'bla'|foo|bar|bla|
'foo' bar 'bla'|foo|bar|bla|
'foo' bar bla|foo|bar|bla|
blurb foo"bar"bar"fasel" baz|blurb|foobarbarfasel|baz|
blurb foo'bar'bar'fasel' baz|blurb|foobarbarfasel|baz|
""||
''||
foo "" bar|foo||bar|
foo '' bar|foo||bar|
foo "" "" "" bar|foo||||bar|
foo '' '' '' bar|foo||||bar|
\"|"|
"\""|"|
"foo\ bar"|foo\ bar|
"foo\\ bar"|foo\ bar|
"foo\\ bar\""|foo\ bar"|
"foo\\" bar\"|foo\|bar"|
"foo\\ bar\" dfadf"|foo\ bar" dfadf|
"foo\\\ bar\" dfadf"|foo\\ bar" dfadf|
"foo\\\x bar\" dfadf"|foo\\x bar" dfadf|
"foo\x bar\" dfadf"|foo\x bar" dfadf|
\'|'|
'foo\ bar'|foo\ bar|
'foo\\ bar'|foo\\ bar|
"foo\\\x bar\" df'a\ 'df"|foo\\x bar" df'a\ 'df|
\"foo|"foo|
\"foo\x|"foox|
"foo\x"|foo\x|
"foo\ "|foo\ |
foo\ xx|foo xx|
foo\ x\x|foo xx|
foo\ x\x\"|foo xx"|
"foo\ x\x"|foo\ x\x|
"foo\ x\x\\"|foo\ x\x\|
"foo\ x\x\\""foobar"|foo\ x\x\foobar|
"foo\ x\x\\"\'"foobar"|foo\ x\x\'foobar|
"foo\ x\x\\"\'"fo'obar"|foo\ x\x\'fo'obar|
"foo\ x\x\\"\'"fo'obar" 'don'\''t'|foo\ x\x\'fo'obar|don't|
"foo\ x\x\\"\'"fo'obar" 'don'\''t' \\|foo\ x\x\'fo'obar|don't|\|
'foo\ bar'|foo\ bar|
'foo\\ bar'|foo\\ bar|
foo\ bar|foo bar|
foo#bar\nbaz|foo|baz|
:-) ;-)|:-)|;-)|
áéíóú|áéíóú|
"""

class ShlexTest(unittest.TestCase):
    def setUp(self):
        self.data = [x.split("|")[:-1]
                     for x in data.splitlines()]
        self.posix_data = [x.split("|")[:-1]
                           for x in posix_data.splitlines()]
        for item in self.data:
            item[0] = item[0].replace(r"\n", "\n")
        for item in self.posix_data:
            item[0] = item[0].replace(r"\n", "\n")

    def splitTest(self, data, comments):
        for i in range(len(data)):
            l = shlex.split(data[i][0], comments=comments)
            self.assertEqual(l, data[i][1:],
                             "%s: %s != %s" %
                             (data[i][0], l, data[i][1:]))

    def oldSplit(self, s):
        ret = []
        lex = shlex.shlex(StringIO(s))
        tok = lex.get_token()
        while tok:
            ret.append(tok)
            tok = lex.get_token()
        return ret

    def testSplitPosix(self):
        """Test data splitting with posix parser"""
        self.splitTest(self.posix_data, comments=True)

    def testCompat(self):
        """Test compatibility interface"""
        for i in range(len(self.data)):
            l = self.oldSplit(self.data[i][0])
            self.assertEqual(l, self.data[i][1:],
                             "%s: %s != %s" %
                             (self.data[i][0], l, self.data[i][1:]))

# Allow this test to be used with old shlex.py
if not getattr(shlex, "split", None):
    for methname in dir(ShlexTest):
        if methname.startswith("test") and methname != "testCompat":
            delattr(ShlexTest, methname)

def test_main():
    test_support.run_unittest(ShlexTest)

if __name__ == "__main__":
    test_main()



# The MIT License (MIT)
#
# Copyright (c) 2014 Richard Moore
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# Why to_bufferable?
# Python 3 is very different from Python 2.x when it comes to strings of text
# and strings of bytes; in Python 3, strings of bytes do not exist, instead to
# represent arbitrary binary data, we must use the "bytes" object. This method
# ensures the object behaves as we need it to.

def to_bufferable(binary):
    return binary

def _get_byte(c):
    return ord(c)

try:
    xrange
except:

    def to_bufferable(binary):
        if isinstance(binary, bytes):
            return binary
        return bytes(ord(b) for b in binary)

    def _get_byte(c):
        return c

def append_PKCS7_padding(data):
    pad = 16 - (len(data) % 16)
    return data + to_bufferable(chr(pad) * pad)

def strip_PKCS7_padding(data):
    if len(data) % 16 != 0:
        raise ValueError("invalid length")

    pad = _get_byte(data[-1])

    if pad > 16:
        raise ValueError("invalid padding byte")

    return data[:-pad]
